@@ -20,11 +20,16 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
 
 #include "gstspideridentity.h"
-
 #include "gstspider.h"
 
+GST_DEBUG_CATEGORY (gst_spider_identity_debug);
+#define GST_CAT_DEFAULT gst_spider_identity_debug
+
 GstElementDetails gst_spider_identity_details = {
   "SpiderIdentity",
   "Generic",
@@ -178,19 +183,24 @@ gst_spider_identity_chain (GstPad *pad, GstBuffer *buf)
       {
 	GstSpiderConnection *conn = (GstSpiderConnection *) list->data;
 	list = g_list_next (list);
-	gst_element_set_eos (GST_ELEMENT (conn->src));
-        gst_pad_push (conn->src->src, GST_BUFFER (gst_event_new (GST_EVENT_EOS)));  
+        if (conn->current != (GstElement *) conn->src) {
+          GST_DEBUG ("sending EOS to unconnected element %s from %s", 
+	       GST_ELEMENT_NAME (conn->src), GST_ELEMENT_NAME (ident));
+          gst_pad_push (conn->src->src, GST_DATA (GST_BUFFER (gst_event_new (GST_EVENT_EOS))));  
+          gst_element_set_eos (GST_ELEMENT (conn->src));
+	}
       }
     }
     /* end hack for current event stuff here */
 
-    gst_pad_event_default (ident->sink, GST_EVENT (buf));
+    gst_pad_event_default (pad, GST_EVENT (buf));
     return;
   }
 
   if ((ident->src != NULL) && (GST_PAD_PEER (ident->src) != NULL)) {
     /* g_print("pushing buffer %p (refcount %d - buffersize %d) to pad %s:%s\n", buf, GST_BUFFER_REFCOUNT (buf), GST_BUFFER_SIZE (buf), GST_DEBUG_PAD_NAME (ident->src)); */
-    gst_pad_push (ident->src, buf);
+    GST_LOG ( "push %p %" G_GINT64_FORMAT, buf, GST_BUFFER_OFFSET (buf));
+    gst_pad_push (ident->src, GST_DATA (buf));
   } else if (GST_IS_BUFFER (buf)) {
     gst_buffer_unref (buf);
   }
@@ -198,8 +208,6 @@ gst_spider_identity_chain (GstPad *pad, GstBuffer *buf)
 GstSpiderIdentity*           
 gst_spider_identity_new_src (gchar *name)
 {
-  //GstSpiderIdentity *ret = (GstSpiderIdentity *) g_object_new (gst_spider_identity_get_type (), NULL);
-  //GST_ELEMENT_NAME (ret) = name;
   GstSpiderIdentity *ret = (GstSpiderIdentity *) gst_element_factory_make ("spideridentity", name);
   /* set the right functions */
   gst_element_set_loop_function (GST_ELEMENT (ret), (GstElementLoopFunction) GST_DEBUG_FUNCPTR (gst_spider_identity_src_loop));
@@ -209,9 +217,6 @@ gst_spider_identity_new_src (gchar *name)
 GstSpiderIdentity*           
 gst_spider_identity_new_sink (gchar *name)
 {
-  //GstSpiderIdentity *ret = (GstSpiderIdentity *) g_object_new (gst_spider_identity_get_type (), NULL);
-  
-  //GST_ELEMENT_NAME (ret) = name;
   GstSpiderIdentity *ret = (GstSpiderIdentity *) gst_element_factory_make ("spideridentity", name);
 
   /* set the right functions */
@@ -272,7 +277,7 @@ gst_spider_identity_request_new_pad  (GstElement *element, GstPadTemplate *templ
     case GST_PAD_SINK:
       if (ident->sink != NULL) break;
       /* sink */
-      GST_DEBUG(0, "element %s requests new sink pad", GST_ELEMENT_NAME(ident));
+      GST_DEBUG ( "element %s requests new sink pad", GST_ELEMENT_NAME(ident));
       ident->sink = gst_pad_new ("sink", GST_PAD_SINK);
       gst_element_add_pad (GST_ELEMENT (ident), ident->sink);
       gst_pad_set_link_function (ident->sink, GST_DEBUG_FUNCPTR (gst_spider_identity_link));
@@ -282,7 +287,7 @@ gst_spider_identity_request_new_pad  (GstElement *element, GstPadTemplate *templ
     case GST_PAD_SRC:
       /* src */
       if (ident->src != NULL) break;
-      GST_DEBUG(0, "element %s requests new src pad", GST_ELEMENT_NAME(ident));
+      GST_DEBUG ( "element %s requests new src pad", GST_ELEMENT_NAME(ident));
       ident->src = gst_pad_new ("src", GST_PAD_SRC);
       gst_element_add_pad (GST_ELEMENT (ident), ident->src);
       gst_pad_set_link_function (ident->src, GST_DEBUG_FUNCPTR (gst_spider_identity_link));
@@ -293,7 +298,7 @@ gst_spider_identity_request_new_pad  (GstElement *element, GstPadTemplate *templ
       break;
   }
   
-  GST_DEBUG(0, "element %s requested a new pad but none could be created", GST_ELEMENT_NAME(ident));
+  GST_DEBUG ( "element %s requested a new pad but none could be created", GST_ELEMENT_NAME(ident));
   return NULL;
 }
 
@@ -354,7 +359,7 @@ gst_spider_identity_start_type_finding (GstSpiderIdentity *ident)
   gchar *name;*/
   gboolean restart = FALSE;
   
-  GST_DEBUG (GST_CAT_AUTOPLUG, "element %s starts typefinding", GST_ELEMENT_NAME(ident));
+  GST_DEBUG ("element %s starts typefinding", GST_ELEMENT_NAME(ident));
   if (GST_STATE (GST_ELEMENT_PARENT (ident)) == GST_STATE_PLAYING)
   {
     gst_element_set_state (GST_ELEMENT (GST_ELEMENT_PARENT (ident)), GST_STATE_PAUSED);
@@ -381,7 +386,7 @@ gst_spider_identity_dumb_loop  (GstSpiderIdentity *ident)
   g_return_if_fail (GST_IS_SPIDER_IDENTITY (ident));
   g_assert (ident->sink != NULL);
 
-  buf = gst_pad_pull (ident->sink);
+  buf = GST_BUFFER (gst_pad_pull (ident->sink));
 
   gst_spider_identity_chain (ident->sink, buf);
 }
@@ -401,7 +406,6 @@ gst_spider_identity_src_loop (GstSpiderIdentity *ident)
     gst_spider_identity_dumb_loop (ident);
     return;
   }
-
   gst_element_interrupt (GST_ELEMENT (ident));
 }
 /* This loop function is only needed when typefinding.
@@ -409,65 +413,29 @@ gst_spider_identity_src_loop (GstSpiderIdentity *ident)
 static void
 gst_spider_identity_sink_loop_type_finding (GstSpiderIdentity *ident)
 {
-  GstBuffer *buf=NULL;
-  GstBuffer *typefindbuf = NULL;
-  gboolean getmorebuf = TRUE;
+  GstBuffer *buf = NULL;
   GList *type_list;
-  gboolean restart_spider = FALSE;
   GstCaps *caps;
+  GstByteStream *bs;
 
-  /* this should possibly be a property */
-  guint bufsizelimit = 8192;
-  
-  /* checks - disable for speed */
-  g_return_if_fail (ident != NULL);
   g_return_if_fail (GST_IS_SPIDER_IDENTITY (ident));
-  
-  while (getmorebuf){
 
-    /* check if our buffer is big enough to do a typefind */
-    if (typefindbuf && GST_BUFFER_SIZE(typefindbuf) >= bufsizelimit){
-      getmorebuf = FALSE;
-      break;
-    }
-    buf = gst_pad_pull (ident->sink);
-  
-    /* if it's an event... */
-    while (GST_IS_EVENT (buf)) {
-      switch (GST_EVENT_TYPE (GST_EVENT (buf))){
-      case GST_EVENT_EOS:
-        getmorebuf = FALSE;
-	/* FIXME Notify the srcs that EOS has happened */
-        gst_pad_event_default (ident->sink, GST_EVENT (buf));
-	break;
-      default:
-        gst_pad_event_default (ident->sink, GST_EVENT (buf));
-        buf = gst_pad_pull (ident->sink);
-        break;
-      }
-      /* handle DISCONT events, please */
-    }
-
-    if (!typefindbuf){
-      typefindbuf = buf;
-    }
-    else {
-      GstBuffer *oldbuf = typefindbuf;
-      typefindbuf = gst_buffer_merge(typefindbuf, buf);
-      gst_buffer_unref (oldbuf);
-      gst_buffer_unref (buf);
-    }
-  }
-  
-  if (!typefindbuf){
+  /* get a bytestream object */
+  bs = gst_bytestream_new (ident->sink);
+  if (gst_bytestream_peek (bs, &buf, 1) != 1 || !buf) {
+    buf = NULL;
+    g_warning ("Failed to read fake buffer - serious idiocy going on here");
     goto end;
+  } else {
+    gst_buffer_unref (buf);
+    buf = NULL;
   }
 
   /* maybe there are already valid caps now? */
   if ((caps = gst_pad_get_caps (ident->sink)) != NULL) {
     goto plug;
   }
-
+  
   /* now do the actual typefinding with the supplied buffer */
   type_list = (GList *) gst_type_get_list ();
     
@@ -479,22 +447,22 @@ gst_spider_identity_sink_loop_type_finding (GstSpiderIdentity *ident)
       GstTypeFactory *factory = GST_TYPE_FACTORY (factories->data);
       GstTypeFindFunc typefindfunc = (GstTypeFindFunc)factory->typefindfunc;
 
-      if (typefindfunc && (caps = typefindfunc (typefindbuf, factory))) {
-
-	if (gst_pad_try_set_caps (ident->sink, caps) <= 0) {
+      GST_DEBUG ("trying typefind function %s", GST_PLUGIN_FEATURE_NAME (factory));
+      if (typefindfunc && (caps = typefindfunc (bs, factory))) {
+        GST_INFO ("typefind function %s found caps", GST_PLUGIN_FEATURE_NAME (factory));
+        if (gst_pad_try_set_caps (ident->src, caps) <= 0) {
           g_warning ("typefind: found type but peer didn't accept it");
-          gst_caps_sink (caps);
-	} else {
+	  gst_caps_sink (caps);
+        } else {
           goto plug;
-        }
+	}
       }
       factories = g_slist_next (factories);
     }
     type_list = g_list_next (type_list);
   }
   gst_element_error(GST_ELEMENT(ident), "Could not find media type", NULL);
-  gst_buffer_unref(typefindbuf);
-  typefindbuf = GST_BUFFER (gst_event_new (GST_EVENT_EOS));
+  buf = GST_BUFFER (gst_event_new (GST_EVENT_EOS));
 
 end:
 
@@ -503,24 +471,21 @@ end:
                                 (GstElementLoopFunction) GST_DEBUG_FUNCPTR (gst_spider_identity_dumb_loop));
   
   /* push the buffer */
-  gst_spider_identity_chain (ident->sink, typefindbuf);
+  gst_spider_identity_chain (ident->sink, buf);
 
+  /* bytestream no longer needed */
+  gst_bytestream_destroy (bs);
+  
   return;
 
 plug:
   gst_caps_debug (caps, "spider starting caps");
   gst_caps_sink (caps);
-  /* pause the autoplugger */
-  if (gst_element_get_state (GST_ELEMENT (GST_ELEMENT_PARENT(ident))) == GST_STATE_PLAYING) {
-    gst_element_set_state (GST_ELEMENT (GST_ELEMENT_PARENT(ident)), GST_STATE_PAUSED);
-    restart_spider = TRUE;
-  }
-  gst_spider_identity_plug (ident);  
 
-  /* restart autoplugger */
-  if (restart_spider){
-    gst_element_set_state (GST_ELEMENT (GST_ELEMENT_PARENT(ident)), GST_STATE_PLAYING);
-  }
+  gst_spider_identity_plug (ident);
+
+  gst_bytestream_read (bs, &buf, bs->listavail);
+
   goto end;
 }
 
@@ -530,7 +495,7 @@ gst_spider_identity_handle_src_event (GstPad *pad, GstEvent *event)
   gboolean res = TRUE;
   GstSpiderIdentity *ident;
 
-  GST_DEBUG (0, "spider_identity src_event\n");
+  GST_DEBUG ( "spider_identity src_event");
 
   ident = GST_SPIDER_IDENTITY (gst_pad_get_parent (pad));
 
@@ -544,5 +509,3 @@ gst_spider_identity_handle_src_event (GstPad *pad, GstEvent *event)
 
   return res;
 }
-
-
