@@ -20,8 +20,14 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
+
 #include "gstaggregator.h"
 
+GST_DEBUG_CATEGORY (gst_aggregator_debug);
+#define GST_CAT_DEFAULT gst_aggregator_debug
 
 GstElementDetails gst_aggregator_details = {
   "Aggregator pipe fitting",
@@ -52,7 +58,7 @@ GST_PAD_TEMPLATE_FACTORY (aggregator_src_factory,
   "sink%d",
   GST_PAD_SINK,
   GST_PAD_REQUEST,
-  NULL			/* no caps */
+  GST_CAPS_ANY
 );
 
 #define GST_TYPE_AGGREGATOR_SCHED (gst_aggregator_sched_get_type())
@@ -79,13 +85,14 @@ static void 	gst_aggregator_init		(GstAggregator *aggregator);
 
 static GstPad* 	gst_aggregator_request_new_pad	(GstElement *element, GstPadTemplate *temp, const
                                                  gchar *unused);
+static void 	gst_aggregator_update_functions	(GstAggregator *aggregator);
 
 static void 	gst_aggregator_set_property 	(GObject *object, guint prop_id, 
 						 const GValue *value, GParamSpec *pspec);
 static void 	gst_aggregator_get_property 	(GObject *object, guint prop_id, 
 						 GValue *value, GParamSpec *pspec);
 
-static void  	gst_aggregator_chain 		(GstPad *pad, GstBuffer *buf);
+static void  	gst_aggregator_chain 		(GstPad *pad, GstData *_data);
 static void 	gst_aggregator_loop 		(GstElement *element);
 
 static GstElementClass *parent_class = NULL;
@@ -153,6 +160,8 @@ gst_aggregator_init (GstAggregator *aggregator)
   aggregator->sinkpads = NULL;
   aggregator->silent = FALSE;
   aggregator->sched = AGGREGATOR_LOOP;
+
+  gst_aggregator_update_functions (aggregator);
 }
 
 static GstPad*
@@ -176,7 +185,9 @@ gst_aggregator_request_new_pad (GstElement *element, GstPadTemplate *templ, cons
   sinkpad = gst_pad_new_from_template (templ, name);
   g_free (name);
   
-  gst_pad_set_chain_function (sinkpad, gst_aggregator_chain);
+  if (!AGGREGATOR_IS_LOOP_BASED (aggregator)) {
+    gst_pad_set_chain_function (sinkpad, gst_aggregator_chain);
+  }
   gst_element_add_pad (GST_ELEMENT (aggregator), sinkpad);
   
   aggregator->sinkpads = g_list_prepend (aggregator->sinkpads, sinkpad);
@@ -275,7 +286,7 @@ gst_aggregator_push (GstAggregator *aggregator, GstPad *pad, GstBuffer *buf, guc
     g_object_notify (G_OBJECT (aggregator), "last_message");
   }
 
-  gst_pad_push (aggregator->srcpad, buf);
+  gst_pad_push (aggregator->srcpad, GST_DATA (buf));
 }
 
 static void 
@@ -290,14 +301,24 @@ gst_aggregator_loop (GstElement *element)
   if (aggregator->sched == AGGREGATOR_LOOP) {
     GList *pads = aggregator->sinkpads;
 
+    /* we'll loop over all pads and try to pull from all
+     * active ones */
     while (pads) {
       GstPad *pad = GST_PAD (pads->data);
       pads = g_list_next (pads);
 
-      buf = gst_pad_pull (pad);
-      debug = "loop";
+      /* we need to check is the pad is usable. IS_USABLE will check
+       * if the pad is linked, if it is enabled (the element is
+       * playing and the app didn't gst_pad_set_enabled (pad, FALSE))
+       * and that the peer pad is also enabled.
+       */
+      if (GST_PAD_IS_USABLE (pad)) {
+        buf = GST_BUFFER (gst_pad_pull (pad));
+        debug = "loop";
 
-      gst_aggregator_push (aggregator, pad, buf, debug);
+	/* then push it forward */
+        gst_aggregator_push (aggregator, pad, buf, debug);
+      }
     }
   }
   else {
@@ -307,7 +328,7 @@ gst_aggregator_loop (GstElement *element)
       debug = "loop_select";
 
       pad = gst_pad_select (aggregator->sinkpads);
-      buf = gst_pad_pull (pad);
+      buf = GST_BUFFER (gst_pad_pull (pad));
 
       gst_aggregator_push (aggregator, pad, buf, debug);
     }
@@ -325,8 +346,9 @@ gst_aggregator_loop (GstElement *element)
  * Chain a buffer on a pad.
  */
 static void 
-gst_aggregator_chain (GstPad *pad, GstBuffer *buf) 
+gst_aggregator_chain (GstPad *pad, GstData *_data) 
 {
+  GstBuffer *buf = GST_BUFFER (_data);
   GstAggregator *aggregator;
 
   g_return_if_fail (pad != NULL);
@@ -346,4 +368,3 @@ gst_aggregator_factory_init (GstElementFactory *factory)
 
   return TRUE;
 }
-
