@@ -24,14 +24,12 @@
 #ifndef __GST_FILESINK_H__
 #define __GST_FILESINK_H__
 
-
-#include <config.h>
 #include <gst/gst.h>
 
 G_BEGIN_DECLS
 
-GstElementDetails gst_filesink_details;
-
+extern GstElementDetails gst_filesink_details;
+GST_DEBUG_CATEGORY_EXTERN(gst_filesink_debug);
 
 #define GST_TYPE_FILESINK \
   (gst_filesink_get_type())
@@ -59,17 +57,14 @@ struct _GstFileSink {
   gchar *filename;
   FILE *file;
 
-  gint filenum;
-
   guint64 data_written;
-  gint maxfilesize;
 };
 
 struct _GstFileSinkClass {
   GstElementClass parent_class;
 
   /* signals */
-  void (*handoff) (GstElement *element,GstPad *pad);
+  void (*handoff) (GstElement *element, GstPad *pad);
 };
 
 GType gst_filesink_get_type(void);
