@@ -30,6 +30,9 @@
 
 G_BEGIN_DECLS
 
+extern GstElementDetails gst_filesrc_details;
+GST_DEBUG_CATEGORY_EXTERN(gst_filesrc_debug);
+
 #define GST_TYPE_FILESRC \
   (gst_filesrc_get_type())
 #define GST_FILESRC(obj) \
@@ -63,7 +66,7 @@ struct _GstFileSrc {
   off_t curoffset;			/* current offset in file*/
   off_t block_size;			/* bytes per read */
   gboolean touch;			/* whether to touch every page */
-  gboolean using_mmap;			/* if we're using mmap to read the file */
+  gboolean using_mmap;
 
   GstBuffer *mapbuf;
   size_t mapsize;
