@@ -22,8 +22,15 @@
 
 
 #include <stdlib.h>
-#include <gstidentity.h>
 
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
+
+#include "gstidentity.h"
+
+GST_DEBUG_CATEGORY (gst_identity_debug);
+#define GST_CAT_DEFAULT gst_identity_debug
 
 GstElementDetails gst_identity_details = {
   "Identity",
@@ -53,6 +60,7 @@ enum {
   ARG_SILENT,
   ARG_LAST_MESSAGE,
   ARG_DUMP,
+  ARG_DELAY_CAPSNEGO,
 };
 
 
@@ -62,7 +70,7 @@ static void gst_identity_init		(GstIdentity *identity);
 static void gst_identity_set_property	(GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec);
 static void gst_identity_get_property	(GObject *object, guint prop_id, GValue *value, GParamSpec *pspec);
 
-static void gst_identity_chain		(GstPad *pad, GstBuffer *buf);
+static void gst_identity_chain		(GstPad *pad, GstData *_data);
 
 static GstElementClass *parent_class = NULL;
 static guint gst_identity_signals[LAST_SIGNAL] = { 0 };
@@ -98,7 +106,8 @@ gst_identity_class_init (GstIdentityClass *klass)
   parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_LOOP_BASED,
-    g_param_spec_boolean ("loop-based", "Loop-based", "Set to TRUE to use loop-based rather than chain-based scheduling",
+    g_param_spec_boolean ("loop-based", "Loop-based", 
+	    		  "Set to TRUE to use loop-based rather than chain-based scheduling",
                           TRUE, G_PARAM_READWRITE)); 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_SLEEP_TIME,
     g_param_spec_uint ("sleep-time", "Sleep time", "Microseconds to sleep between processing",
@@ -119,7 +128,10 @@ gst_identity_class_init (GstIdentityClass *klass)
     g_param_spec_string ("last-message", "last-message", "last-message",
                          NULL, G_PARAM_READABLE)); 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_DUMP,
-    g_param_spec_boolean("dump","Dump","Dump buffer contents",
+    g_param_spec_boolean("dump", "Dump", "Dump buffer contents",
+                         FALSE, G_PARAM_READWRITE));
+  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_DELAY_CAPSNEGO,
+    g_param_spec_boolean("delay_capsnego", "Delay Caps Nego", "Delay capsnegotiation to loop/chain function",
                          FALSE, G_PARAM_READWRITE));
 
   gst_identity_signals[SIGNAL_HANDOFF] =
@@ -149,6 +161,11 @@ gst_identity_getcaps (GstPad *pad, GstCaps *caps)
   GstPad *otherpad;
   
   identity = GST_IDENTITY (gst_pad_get_parent (pad));
+
+  if (identity->delay_capsnego) {
+    return NULL;
+  }
+
   otherpad = (pad == identity->srcpad ? identity->sinkpad : identity->srcpad);
 
   return gst_pad_get_allowed_caps (otherpad);
@@ -158,13 +175,23 @@ static GstPadLinkReturn
 gst_identity_link (GstPad *pad, GstCaps *caps)
 {
   GstIdentity *identity;
-  GstPad *otherpad;
   
   identity = GST_IDENTITY (gst_pad_get_parent (pad));
-  otherpad = (pad == identity->srcpad ? identity->sinkpad : identity->srcpad);
 
-  if (GST_CAPS_IS_FIXED (caps))
-    return gst_pad_try_set_caps (otherpad, caps);
+  if (GST_CAPS_IS_FIXED (caps)) {
+    if (identity->delay_capsnego && GST_PAD_IS_SINK (pad)) {
+      identity->srccaps = gst_caps_ref (caps);
+
+      return GST_PAD_LINK_OK;
+    }
+    else {
+      GstPad *otherpad;
+      
+      otherpad = (pad == identity->srcpad ? identity->sinkpad : identity->srcpad);
+
+      return gst_pad_try_set_caps (otherpad, caps);
+    }
+  }
   else
     return GST_PAD_LINK_DELAYED;
 }
@@ -192,11 +219,14 @@ gst_identity_init (GstIdentity *identity)
   identity->silent = FALSE;
   identity->dump = FALSE;
   identity->last_message = NULL;
+  identity->delay_capsnego = FALSE;
+  identity->srccaps = NULL;
 }
 
 static void 
-gst_identity_chain (GstPad *pad, GstBuffer *buf) 
+gst_identity_chain (GstPad *pad, GstData *_data) 
 {
+  GstBuffer *buf = GST_BUFFER (_data);
   GstIdentity *identity;
   guint i;
 
@@ -206,6 +236,16 @@ gst_identity_chain (GstPad *pad, GstBuffer *buf)
 
   identity = GST_IDENTITY (gst_pad_get_parent (pad));
 
+  if (identity->delay_capsnego && identity->srccaps) {
+    if (gst_pad_try_set_caps (identity->srcpad, identity->srccaps) <= 0) {
+      if (!gst_pad_recover_caps_error (identity->srcpad, identity->srccaps)) {
+        gst_buffer_unref (buf);
+        return;
+      }
+    }
+    identity->srccaps = NULL;
+  }
+
   if (identity->error_after >= 0) {
     identity->error_after--;
     if (identity->error_after == 0) {
@@ -247,10 +287,10 @@ gst_identity_chain (GstPad *pad, GstBuffer *buf)
     if (i>1) 
       gst_buffer_ref (buf);
 
-    gst_pad_push (identity->srcpad, buf);
+    gst_pad_push (identity->srcpad, GST_DATA (buf));
 
     if (identity->sleep_time)
-      usleep (identity->sleep_time);
+      g_usleep (identity->sleep_time);
   }
 }
 
@@ -265,7 +305,7 @@ gst_identity_loop (GstElement *element)
 
   identity = GST_IDENTITY (element);
   
-  buf = gst_pad_pull (identity->sinkpad);
+  buf = GST_BUFFER (gst_pad_pull (identity->sinkpad));
   if (GST_IS_EVENT (buf)) {
     GstEvent *event = GST_EVENT (buf);
 
@@ -277,7 +317,7 @@ gst_identity_loop (GstElement *element)
     }
   }
   else {
-    gst_identity_chain (identity->sinkpad, buf);
+    gst_identity_chain (identity->sinkpad, GST_DATA (buf));
   }
 }
 
@@ -315,6 +355,9 @@ gst_identity_set_property (GObject *object, guint prop_id, const GValue *value,
     case ARG_DUMP:
       identity->dump = g_value_get_boolean (value);
       break;
+    case ARG_DELAY_CAPSNEGO:
+      identity->delay_capsnego = g_value_get_boolean (value);
+      break;
     case ARG_ERROR_AFTER:
       identity->error_after = g_value_get_int (value);
       break;
@@ -356,6 +399,9 @@ static void gst_identity_get_property(GObject *object, guint prop_id, GValue *va
     case ARG_DUMP:
       g_value_set_boolean (value, identity->dump);
       break;
+    case ARG_DELAY_CAPSNEGO:
+      g_value_set_boolean (value, identity->delay_capsnego);
+      break;
     case ARG_LAST_MESSAGE:
       g_value_set_string (value, identity->last_message);
       break;
