@@ -25,11 +25,16 @@
 
 #include <string.h>
 
-#include <config.h>
-#include <gst/gst.h>
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
 
+#include <gst/gst.h>
 #include "gstmd5sink.h"
 
+GST_DEBUG_CATEGORY (gst_md5sink_debug);
+#define GST_CAT_DEFAULT gst_md5sink_debug
+
 /* MD5Sink signals and args */
 enum {
   /* FILL ME */
@@ -46,7 +51,7 @@ GST_PAD_TEMPLATE_FACTORY (md5_sink_factory,
   "sink",
   GST_PAD_SINK,
   GST_PAD_ALWAYS,
-  NULL                  /* no caps */
+  GST_CAPS_ANY
 );
 
 /* GObject stuff */
@@ -58,7 +63,7 @@ static void			gst_md5sink_init		(GstMD5Sink *md5sink);
 static void			gst_md5sink_get_property	(GObject *object, guint prop_id, 
 								 GValue *value, GParamSpec *pspec);
 
-static void			gst_md5sink_chain		(GstPad *pad, GstBuffer *buf);
+static void			gst_md5sink_chain		(GstPad *pad, GstData *_data);
 static GstElementStateReturn	gst_md5sink_change_state	(GstElement *element);
 
 /* variables */
@@ -484,8 +489,9 @@ gst_md5sink_get_property (GObject *object, guint prop_id, GValue *value, GParamS
 }
 
 static void 
-gst_md5sink_chain (GstPad *pad, GstBuffer *buf) 
+gst_md5sink_chain (GstPad *pad, GstData *_data) 
 {
+  GstBuffer *buf = GST_BUFFER (_data);
   GstMD5Sink *md5sink;
 
   g_return_if_fail (pad != NULL);
@@ -519,4 +525,3 @@ gst_md5sink_factory_init (GstElementFactory *factory)
 
   return TRUE;
 }
-
