@@ -21,8 +21,14 @@
  */
 
 
-#include <gststatistics.h>
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
 
+#include "gststatistics.h"
+
+GST_DEBUG_CATEGORY (gst_statistics_debug);
+#define GST_CAT_DEFAULT gst_statistics_debug
 
 GstElementDetails gst_statistics_details = {
   "Statistics",
@@ -61,7 +67,7 @@ static void gst_statistics_init		(GstStatistics *statistics);
 static void gst_statistics_set_property	(GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec);
 static void gst_statistics_get_property	(GObject *object, guint prop_id, GValue *value, GParamSpec *pspec);
 
-static void gst_statistics_chain		(GstPad *pad, GstBuffer *buf);
+static void gst_statistics_chain		(GstPad *pad, GstData *_data);
 static void gst_statistics_reset		(GstStatistics *statistics);
 static void gst_statistics_print		(GstStatistics *statistics);
 
@@ -250,8 +256,9 @@ gst_statistics_print (GstStatistics *statistics)
 }
 
 static void 
-gst_statistics_chain (GstPad *pad, GstBuffer *buf) 
+gst_statistics_chain (GstPad *pad, GstData *_data) 
 {
+  GstBuffer *buf = GST_BUFFER (_data);
   GstStatistics *statistics;
   gboolean update = FALSE;
 
@@ -299,15 +306,15 @@ gst_statistics_chain (GstPad *pad, GstBuffer *buf)
 
   if (update) {
     if (statistics->update) {
-      GST_DEBUG_ELEMENT (GST_CAT_DATAFLOW, statistics, "pre update emit\n");
+      GST_DEBUG ("[%s]: pre update emit", GST_ELEMENT_NAME (statistics));
       g_signal_emit (G_OBJECT (statistics), gst_statistics_signals[SIGNAL_UPDATE], 0);
-      GST_DEBUG_ELEMENT (GST_CAT_DATAFLOW, statistics, "post update emit\n");
+      GST_DEBUG ("[%s]: post update emit", GST_ELEMENT_NAME (statistics));
     }
     if (!statistics->silent) {
       gst_statistics_print(statistics);
     }
   }
-  gst_pad_push (statistics->srcpad, buf);
+  gst_pad_push (statistics->srcpad, GST_DATA (buf));
 }
 
 static void 
