@@ -35,10 +35,10 @@
 #include <gst/gstpad.h>
 #include <gst/gstbuffer.h>
 #include <gst/gstbufferpool-default.h>
-#include <gst/gstcache.h>
 #include <gst/gstcpu.h>
 #include <gst/gstelement.h>
 #include <gst/gstbin.h>
+#include <gst/gstindex.h>
 #include <gst/gstpipeline.h>
 #include <gst/gstthread.h>
 #include <gst/gsttype.h>
@@ -46,6 +46,8 @@
 #include <gst/gstcaps.h>
 #include <gst/gstprops.h>
 #include <gst/gstplugin.h>
+#include <gst/gsturi.h>
+#include <gst/gsturitype.h>
 #include <gst/gstutils.h>
 #include <gst/gsttrace.h>
 #include <gst/gstxml.h>
@@ -53,18 +55,25 @@
 #include <gst/gstevent.h>
 #include <gst/gstclock.h>
 #include <gst/gstsystemclock.h>
+#include <gst/gstinterface.h>
 
 #include <gst/gstparse.h>
 #include <gst/gstregistry.h>
-#include <gst/gstextratypes.h>
+#include <gst/gstregistrypool.h>
 #include <gst/gstenumtypes.h>
 
+/* API compatibility stuff */
+#include <gst/gstcompat.h>
+
 G_BEGIN_DECLS
 
 /* initialize GST */
 void		gst_init			(int *argc, char **argv[]);
 gboolean	gst_init_check			(int *argc, char **argv[]);
-gboolean	gst_init_with_popt_table	(int *argc, char **argv[],
+void		gst_init_with_popt_table	(int *argc, char **argv[],
+						 const struct poptOption
+						 *popt_options);
+gboolean	gst_init_check_with_popt_table	(int *argc, char **argv[],
 						 const struct poptOption
 						 *popt_options);
 const struct
