@@ -77,20 +77,17 @@ struct _GstBin {
 
   GstElementState child_states[GST_NUM_STATES];
 
-  GstBinPrePostIterateFunction  pre_iterate_func;
-  GstBinPrePostIterateFunction post_iterate_func;
-  gpointer	            pre_iterate_data;
-  gpointer	            post_iterate_data;
-
-  gpointer	 dummy[8];
+  GST_OBJECT_PADDING
 };
 
 struct _GstBinClass {
   GstElementClass parent_class;
 
   /* vtable */
-  void		(*add_element)		(GstBin *bin, GstElement);
-  void		(*remove_element)	(GstBin *bin, GstElement);
+  void		(*add_element)		(GstBin *bin, GstElement *element);
+  void		(*remove_element)	(GstBin *bin, GstElement *element);
+  void		(*child_state_change)	(GstBin *bin, GstElementState oldstate, 
+					 GstElementState newstate, GstElement *element);
 
   /* run a full iteration of operation */
   gboolean	(*iterate)		(GstBin *bin);
@@ -99,17 +96,17 @@ struct _GstBinClass {
   void		(*element_added)	(GstBin *bin, GstElement *child);
   void		(*element_removed)	(GstBin *bin, GstElement *child);
 
-  gpointer	 dummy[8];
+  GST_CLASS_PADDING
 };
 
 GType		gst_bin_get_type		(void);
 GstElement*	gst_bin_new			(const gchar *name);
-#define		gst_bin_destroy(bin)		gst_object_destroy(GST_OBJECT(bin))
 
 /* add and remove elements from the bin */
 void		gst_bin_add			(GstBin *bin, GstElement *element);
 void		gst_bin_add_many 		(GstBin *bin, GstElement *element_1, ...);
 void		gst_bin_remove			(GstBin *bin, GstElement *element);
+void		gst_bin_remove_many		(GstBin *bin, GstElement *element_1, ...);
 
 /* retrieve a single element or the list of children */
 GstElement*	gst_bin_get_by_name		(GstBin *bin, const gchar *name);
@@ -122,6 +119,8 @@ void		gst_bin_use_clock		(GstBin *bin, GstClock *clock);
 GstClock*	gst_bin_get_clock		(GstBin *bin);
 void		gst_bin_auto_clock		(GstBin *bin);
 
+GstElementStateReturn gst_bin_sync_children_state (GstBin *bin);
+
 /* internal */
 /* one of our childs signaled a state change */
 void 		gst_bin_child_state_change 		(GstBin *bin, GstElementState oldstate, 
@@ -138,4 +137,3 @@ G_END_DECLS
 
 
 #endif /* __GST_BIN_H__ */
-
