@@ -20,22 +20,25 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/* this file makes too much noise for most debugging sessions */
-#define GST_DEBUG_FORCE_DISABLE
 #include "gst_private.h"
 
 #include "gstatomic_impl.h"
 #include "gstdata_private.h"
 #include "gstbuffer.h"
 #include "gstmemchunk.h"
-#include "gstlog.h"
+#include "gstinfo.h"
 #include "gstbufferpool-default.h"
 
 GType _gst_buffer_type;
 GType _gst_buffer_pool_type;
 
-static gint _gst_buffer_live;
-static gint _gst_buffer_pool_live;
+#ifndef GST_DISABLE_TRACE
+/* #define GST_WITH_ALLOC_TRACE  */
+#include "gsttrace.h"
+
+static GstAllocTrace *_gst_buffer_trace;
+static GstAllocTrace *_gst_buffer_pool_trace;
+#endif
 
 static GstMemChunk *chunk;
 
@@ -50,27 +53,21 @@ _gst_buffer_initialize (void)
 		            (GBoxedCopyFunc) gst_data_ref,
 			    (GBoxedFreeFunc) gst_data_unref);
 
-  _gst_buffer_live = 0;
-  _gst_buffer_pool_live = 0;
+#ifndef GST_DISABLE_TRACE
+  _gst_buffer_trace = gst_alloc_trace_register (GST_BUFFER_TRACE_NAME);
+  _gst_buffer_pool_trace = gst_alloc_trace_register (GST_BUFFER_POOL_TRACE_NAME);
+#endif
 
   chunk = gst_mem_chunk_new ("GstBufferChunk", sizeof (GstBuffer), 
                              sizeof (GstBuffer) * 200, 0);
 
-  GST_INFO (GST_CAT_BUFFER, "Buffers are initialized now");
+  GST_CAT_INFO (GST_CAT_BUFFER, "Buffers are initialized now");
 }
 
-/**
- * gst_buffer_print_stats:
- *
- * Logs statistics about live buffers (using g_log).
- */
-void
-gst_buffer_print_stats (void)
+GType
+gst_buffer_get_type (void)
 {
-  g_log (g_log_domain_gstreamer, G_LOG_LEVEL_INFO, "%d live buffer(s)", 
-                                 _gst_buffer_live);
-  g_log (g_log_domain_gstreamer, G_LOG_LEVEL_INFO, "%d live bufferpool(s)", 
-                                 _gst_buffer_pool_live);
+  return _gst_buffer_type;
 }
 
 static void
@@ -94,7 +91,9 @@ _gst_buffer_sub_free (GstBuffer *buffer)
   _GST_DATA_DISPOSE (GST_DATA (buffer));
   
   gst_mem_chunk_free (chunk, GST_DATA (buffer));
-  _gst_buffer_live--;
+#ifndef GST_DISABLE_TRACE
+  gst_alloc_trace_free (_gst_buffer_trace, buffer);
+#endif
 }
 
 /**
@@ -121,7 +120,9 @@ gst_buffer_default_free (GstBuffer *buffer)
   _GST_DATA_DISPOSE (GST_DATA (buffer));
 
   gst_mem_chunk_free (chunk, GST_DATA (buffer));
-  _gst_buffer_live--;
+#ifndef GST_DISABLE_TRACE
+  gst_alloc_trace_free (_gst_buffer_trace, buffer);
+#endif
 }
 
 static GstBuffer*
@@ -147,15 +148,27 @@ gst_buffer_default_copy (GstBuffer *buffer)
   g_return_val_if_fail (buffer != NULL, NULL);
 
   /* create a fresh new buffer */
-  copy = gst_buffer_new ();
+  copy = gst_mem_chunk_alloc (chunk);
+#ifndef GST_DISABLE_TRACE
+  gst_alloc_trace_new (_gst_buffer_trace, copy);
+#endif
+
+  _GST_DATA_INIT (GST_DATA (copy), 
+		  _gst_buffer_type,
+		  0,
+		  (GstDataFreeFunction) gst_buffer_default_free,
+		  (GstDataCopyFunction) gst_buffer_default_copy);
 
   /* we simply copy everything from our parent */
-  GST_BUFFER_DATA (copy) 	= g_memdup (GST_BUFFER_DATA (buffer), 
-                                            GST_BUFFER_SIZE (buffer));
-  GST_BUFFER_SIZE (copy)  	= GST_BUFFER_SIZE (buffer);
-  GST_BUFFER_MAXSIZE (copy) 	= GST_BUFFER_MAXSIZE (buffer);
-  GST_BUFFER_TIMESTAMP (copy) 	= GST_BUFFER_TIMESTAMP (buffer);
-  GST_BUFFER_OFFSET (copy) 	= GST_BUFFER_OFFSET (buffer);
+  GST_BUFFER_DATA (copy) 	 = g_memdup (GST_BUFFER_DATA (buffer), 
+                                             GST_BUFFER_SIZE (buffer));
+  GST_BUFFER_SIZE (copy)  	 = GST_BUFFER_SIZE (buffer);
+  GST_BUFFER_MAXSIZE (copy) 	 = GST_BUFFER_SIZE (buffer);
+  GST_BUFFER_TIMESTAMP (copy) 	 = GST_BUFFER_TIMESTAMP (buffer);
+  GST_BUFFER_DURATION (copy) 	 = GST_BUFFER_DURATION (buffer);
+  GST_BUFFER_OFFSET (copy) 	 = GST_BUFFER_OFFSET (buffer);
+  GST_BUFFER_BUFFERPOOL (copy)   = NULL;
+  GST_BUFFER_POOL_PRIVATE (copy) = NULL;
 
   return copy;
 }
@@ -170,21 +183,31 @@ gst_buffer_default_copy (GstBuffer *buffer)
 GstBuffer*
 gst_buffer_new (void)
 {
-  GstBuffer *new;
+  GstBuffer *newbuf;
   
-  new = gst_mem_chunk_alloc0 (chunk);
-  _gst_buffer_live++;
+  newbuf = gst_mem_chunk_alloc (chunk);
+#ifndef GST_DISABLE_TRACE
+  gst_alloc_trace_new (_gst_buffer_trace, newbuf);
+#endif
 
-  _GST_DATA_INIT (GST_DATA (new), 
+  GST_CAT_LOG (GST_CAT_BUFFER, "new %p", newbuf);
+
+  _GST_DATA_INIT (GST_DATA (newbuf), 
 		  _gst_buffer_type,
 		  0,
 		  (GstDataFreeFunction) gst_buffer_default_free,
 		  (GstDataCopyFunction) gst_buffer_default_copy);
 
-  GST_BUFFER_BUFFERPOOL (new) = NULL;
-  GST_BUFFER_POOL_PRIVATE (new) = NULL;
+  GST_BUFFER_DATA (newbuf)         = NULL;
+  GST_BUFFER_SIZE (newbuf)         = 0;
+  GST_BUFFER_MAXSIZE (newbuf)      = GST_BUFFER_MAXSIZE_NONE;
+  GST_BUFFER_TIMESTAMP (newbuf)    = GST_CLOCK_TIME_NONE;
+  GST_BUFFER_DURATION (newbuf)     = GST_CLOCK_TIME_NONE;
+  GST_BUFFER_OFFSET (newbuf)       = GST_BUFFER_OFFSET_NONE;
+  GST_BUFFER_BUFFERPOOL (newbuf)   = NULL;
+  GST_BUFFER_POOL_PRIVATE (newbuf) = NULL;
 
-  return new;
+  return newbuf;
 }
 
 /**
@@ -198,14 +221,15 @@ gst_buffer_new (void)
 GstBuffer*
 gst_buffer_new_and_alloc (guint size)
 {
-  GstBuffer *new;
+  GstBuffer *newbuf;
 
-  new = gst_buffer_new ();
+  newbuf = gst_buffer_new ();
 
-  GST_BUFFER_DATA (new) = g_malloc (size);
-  GST_BUFFER_SIZE (new) = size;
+  GST_BUFFER_DATA (newbuf)    = g_malloc (size);
+  GST_BUFFER_SIZE (newbuf)    = size;
+  GST_BUFFER_MAXSIZE (newbuf) = size;
 
-  return new;
+  return newbuf;
 }
 
 /**
@@ -227,12 +251,13 @@ gst_buffer_new_from_pool (GstBufferPool *pool,
   
   g_return_val_if_fail (pool != NULL, NULL);
 
+  gst_data_ref (GST_DATA (pool));
+
   buffer = pool->buffer_new (pool, offset, size, pool->user_data);
   if (!buffer)
     return NULL;
 
   GST_BUFFER_BUFFERPOOL (buffer) = pool;
-  gst_data_ref (GST_DATA (pool));
 
   /* override the buffer refcount functions with those from the pool (if any) */
   if (pool->buffer_free)
@@ -251,6 +276,9 @@ gst_buffer_new_from_pool (GstBufferPool *pool,
  *
  * Creates a sub-buffer from the parent at a given offset.
  * This sub-buffer uses the actual memory space of the parent buffer.
+ * This function will copy the offset and timestamp field when the 
+ * offset is 0, else they are set to _NONE.
+ * The duration field of the new buffer are set to GST_CLOCK_TIME_NONE.
  *
  * Returns: the new #GstBuffer, or NULL if there was an error.
  */
@@ -259,7 +287,6 @@ gst_buffer_create_sub (GstBuffer *parent, guint offset, guint size)
 {
   GstBuffer *buffer;
   gpointer buffer_data;
-  guint64 parent_offset;
 	      
   g_return_val_if_fail (parent != NULL, NULL);
   g_return_val_if_fail (GST_BUFFER_REFCOUNT_VALUE (parent) > 0, NULL);
@@ -268,7 +295,6 @@ gst_buffer_create_sub (GstBuffer *parent, guint offset, guint size)
 
   /* remember the data for the new buffer */
   buffer_data = parent->data + offset;
-  parent_offset = GST_BUFFER_OFFSET (parent);
   /* make sure we're child not child from a child buffer */
   while (GST_BUFFER_FLAG_IS_SET (parent, GST_BUFFER_SUBBUFFER)) {
     parent = GST_BUFFER (parent->pool_private);
@@ -276,12 +302,15 @@ gst_buffer_create_sub (GstBuffer *parent, guint offset, guint size)
   /* ref the real parent */
   gst_data_ref (GST_DATA (parent));
   /* make sure nobody overwrites data in the parent */
-  if (!GST_DATA_IS_READONLY (parent))
-    GST_DATA_FLAG_SET(parent, GST_DATA_READONLY);
+  GST_DATA_FLAG_SET (parent, GST_DATA_READONLY);
 
   /* create the new buffer */
-  buffer = gst_mem_chunk_alloc0 (chunk);
-  _gst_buffer_live++;
+  buffer = gst_mem_chunk_alloc (chunk);
+#ifndef GST_DISABLE_TRACE
+  gst_alloc_trace_new (_gst_buffer_trace, buffer);
+#endif
+
+  GST_CAT_LOG (GST_CAT_BUFFER, "new subbuffer %p", buffer);
 
   /* make sure nobody overwrites data in the new buffer 
    * by setting the READONLY flag */
@@ -292,14 +321,24 @@ gst_buffer_create_sub (GstBuffer *parent, guint offset, guint size)
 		  (GstDataFreeFunction) _gst_buffer_sub_free,
 		  (GstDataCopyFunction) gst_buffer_default_copy);
 
-  GST_BUFFER_OFFSET (buffer) = parent_offset + offset;
-  GST_BUFFER_TIMESTAMP (buffer) = -1;
-  GST_BUFFER_BUFFERPOOL (buffer) = NULL;
-  GST_BUFFER_POOL_PRIVATE (buffer) = parent;
-
   /* set the right values in the child */
-  buffer->data = buffer_data;
-  buffer->size = size;
+  GST_BUFFER_DATA (buffer)         = buffer_data;
+  GST_BUFFER_SIZE (buffer)         = size;
+  GST_BUFFER_MAXSIZE (buffer)      = size;
+  GST_BUFFER_BUFFERPOOL (buffer)   = NULL;
+  GST_BUFFER_POOL_PRIVATE (buffer) = parent;
+  /* we can copy the timestamp and offset if the new buffer starts at
+   * offset 0 */
+  if (offset == 0) {
+    GST_BUFFER_TIMESTAMP (buffer)    = GST_BUFFER_TIMESTAMP (parent);
+    GST_BUFFER_OFFSET (buffer)       = GST_BUFFER_OFFSET (parent);
+  }
+  else {
+    GST_BUFFER_TIMESTAMP (buffer)    = GST_CLOCK_TIME_NONE;
+    GST_BUFFER_OFFSET (buffer)       = GST_BUFFER_OFFSET_NONE;
+  }
+
+  GST_BUFFER_DURATION (buffer)     = GST_CLOCK_TIME_NONE;
 
   return buffer;
 }
@@ -384,6 +423,7 @@ gst_buffer_span (GstBuffer *buf1, guint32 offset, GstBuffer *buf2, guint32 len)
   g_return_val_if_fail (GST_BUFFER_REFCOUNT_VALUE (buf1) > 0, NULL);
   g_return_val_if_fail (GST_BUFFER_REFCOUNT_VALUE (buf2) > 0, NULL);
   g_return_val_if_fail (len > 0, NULL);
+  g_return_val_if_fail (len <= buf1->size + buf2->size - offset, NULL);
 
   /* if the two buffers have the same parent and are adjacent */
   if (gst_buffer_is_span_fast (buf1, buf2)) {
@@ -393,27 +433,43 @@ gst_buffer_span (GstBuffer *buf1, guint32 offset, GstBuffer *buf2, guint32 len)
 	                            buf1->data - parent->data + offset, len);
   }
   else {
-    GST_DEBUG (GST_CAT_BUFFER, "slow path taken while spanning buffers %p and %p", 
+    GST_CAT_DEBUG (GST_CAT_BUFFER, "slow path taken while spanning buffers %p and %p", 
 	       buf1, buf2);
     /* otherwise we simply have to brute-force copy the buffers */
     newbuf = gst_buffer_new_and_alloc (len);
 
-    /* copy relevant stuff from data struct of buffer1 */
-    GST_BUFFER_OFFSET (newbuf) = GST_BUFFER_OFFSET (buf1) + offset;
-
     /* copy the first buffer's data across */
     memcpy (newbuf->data, buf1->data + offset, buf1->size - offset);
     /* copy the second buffer's data across */
     memcpy (newbuf->data + (buf1->size - offset), buf2->data, 
 	    len - (buf1->size - offset));
+    /* if the offset is 0, the new buffer has the same timestamp as buf1 */
+    if (offset == 0) {
+      GST_BUFFER_OFFSET (newbuf) = GST_BUFFER_OFFSET (buf1);
+      GST_BUFFER_TIMESTAMP (newbuf) = GST_BUFFER_TIMESTAMP (buf1);
+    }
+  }
+  /* if we completely merged the two buffers (appended), we can
+   * calculate the duration too. Also make sure we's not messing with
+   * invalid DURATIONS */
+  if (offset == 0 && buf1->size + buf2->size == len &&
+      GST_BUFFER_DURATION_IS_VALID (buf1) &&
+      GST_BUFFER_DURATION_IS_VALID (buf2)) 
+  {
+    /* add duration */
+    GST_BUFFER_DURATION (newbuf) = GST_BUFFER_DURATION (buf1) + 
+                                   GST_BUFFER_DURATION (buf2);
   }
-  /* if the offset is 0, the new buffer has the same timestamp as buf1 */
-  if (offset == 0)
-    GST_BUFFER_TIMESTAMP (newbuf) = GST_BUFFER_TIMESTAMP (buf1);
 
   return newbuf;
 }
 
+GType
+gst_buffer_pool_get_type (void)
+{
+  return _gst_buffer_pool_type;
+}
+
 /**
  * gst_buffer_pool_default_free:
  * @pool: a #GstBufferPool to free.
@@ -427,7 +483,9 @@ gst_buffer_pool_default_free (GstBufferPool *pool)
 
   _GST_DATA_DISPOSE (GST_DATA (pool));
   g_free (pool);
-  _gst_buffer_pool_live--;
+#ifndef GST_DISABLE_TRACE
+  gst_alloc_trace_free (_gst_buffer_pool_trace, pool);
+#endif
 }
 
 /** 
@@ -460,9 +518,11 @@ gst_buffer_pool_new (GstDataFreeFunction free,
   g_return_val_if_fail (buffer_new != NULL, NULL);
 
   pool = g_new0 (GstBufferPool, 1);
-  _gst_buffer_pool_live++;
+#ifndef GST_DISABLE_TRACE
+  gst_alloc_trace_new (_gst_buffer_pool_trace, pool);
+#endif
 
-  GST_DEBUG (GST_CAT_BUFFER, "allocating new buffer pool %p\n", pool);
+  GST_CAT_DEBUG (GST_CAT_BUFFER, "allocating new buffer pool %p\n", pool);
         
   /* init data struct */
   _GST_DATA_INIT (GST_DATA (pool), 
@@ -542,4 +602,3 @@ gst_buffer_pool_get_user_data (GstBufferPool *pool)
 
   return pool->user_data;
 }
-
