@@ -20,49 +20,154 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/* #define GST_DEBUG_ENABLED */
 #include "gst_private.h"
 
 #include "gstcaps.h"
 #include "gsttype.h"
 #include "gstmemchunk.h"
-#include "gstlog.h"
+#include "gstinfo.h"
+
+#ifndef GST_DISABLE_TRACE
+/* #define GST_WITH_ALLOC_TRACE */
+#include "gsttrace.h"
+
+static GstAllocTrace *_gst_caps_trace;
+#endif
 
 static GstMemChunk *_gst_caps_chunk;
 
 GType _gst_caps_type;
 
+extern GstProps *	__gst_props_from_string_func		(gchar *s, gchar **end, gboolean caps);
+extern gboolean		__gst_props_parse_string		(gchar *r, gchar **end, gchar **next);
+
+/* transform functions */
+static void		gst_caps_transform_to_string		(const GValue *src_value, GValue *dest_value);
+
+static void		gst_caps_destroy			(GstCaps *caps);
+
+
 static void
-transform_func (const GValue *src_value,
-		GValue *dest_value)
+gst_caps_transform_to_string (const GValue *src_value, GValue *dest_value)
 {
   GstCaps *caps = g_value_peek_pointer (src_value);
-  GString *result = g_string_new (""); 
+  dest_value->data[0].v_pointer = gst_caps_to_string (caps);
+}
+/**
+ * gst_caps_to_string:
+ * caps: the caps to convert to a string
+ *
+ * Converts a #GstCaps into a readable format. This is mainly intended for
+ * debugging purposes. You have to free the string using g_free.
+ * A string converted with #gst_caps_to_string can always be converted back to
+ * its caps representation using #gst_caps_from_string.
+ *
+ * Returns: A newly allocated string
+ */
+gchar *
+gst_caps_to_string (GstCaps *caps)
+{
+  gchar *ret;
+  GString *result;
+
+  result = g_string_new ("");
+
+  if(caps==NULL){
+    g_string_append(result, "NULL");
+  }
 
-  g_string_append_printf (result, "(GstCaps *) ");
-		  
   while (caps) {
     gchar *props;
     GValue value = { 0, }; /* the important thing is that value.type = 0 */
     
-    g_string_append_printf (result,
-	          "{ %s; ", gst_caps_get_mime (caps));
+    g_string_append_printf (result, "\"%s\"", gst_caps_get_mime (caps));
 
-    g_value_init (&value, GST_TYPE_PROPS);
-    g_value_set_boxed  (&value, caps->properties);
-    props = g_strdup_value_contents (&value);
+    if (caps->properties) {
+      g_value_init (&value, GST_TYPE_PROPS);
+      g_value_set_boxed  (&value, caps->properties);
+      props = g_strdup_value_contents (&value);
 
-    g_value_unset (&value);
-    g_string_append (result, props);
-    g_free (props);
+      g_value_unset (&value);
+      g_string_append (result, ", ");
+      g_string_append (result, props);
+      g_free (props);
+    }
 
     caps = caps->next;
-    g_string_append_printf (result, " }%s", (caps ? ", " : ""));
+    if (caps)
+      g_string_append (result, "; ");
   }
-  dest_value->data[0].v_pointer = result->str;
+  ret = result->str;
   g_string_free (result, FALSE);
+  return ret;
 }
 
+static GstCaps *
+gst_caps_from_string_func (gchar *r)
+{
+  gchar *mime, *w;
+  GstCaps *caps, *append;
+  GstProps *props = NULL;
+
+  mime = r;
+  if (!__gst_props_parse_string (r, &w, &r)) goto error;
+    
+  if (*r == '\0') goto found;
+  if (*r++ != ',') goto error;
+  while (g_ascii_isspace (*r)) r++;
+    
+  props = __gst_props_from_string_func (r, &r, TRUE);
+  if (!props) goto error;
+
+found:
+  *w = '\0';
+  if (*mime == '\0') {
+    gst_props_unref (props);
+    goto error;
+  }  
+  caps = gst_caps_new ("parsed caps", mime, props);
+  if (*r == '\0')
+    return caps;
+  
+  while (g_ascii_isspace (*r)) r++;
+  if (*r == ';') {
+    r++;
+    while (g_ascii_isspace (*r)) r++;
+    append = gst_caps_from_string_func (r);
+    if (!append) {
+      gst_caps_unref (caps);
+      goto error;
+    }
+    gst_caps_append (caps, append);
+  }
+
+  return caps;
+
+error:
+  return NULL;
+}
+/**
+ * gst_caps_from_string:
+ * str: the str to convert into caps
+ *
+ * Tries to convert a string into a #GstCaps. This is mainly intended for
+ * debugging purposes. The returned caps are floating.
+ *
+ * Returns: A floating caps or NULL if the string couldn't be converted
+ */
+GstCaps *
+gst_caps_from_string (gchar *str)
+{
+  gchar *s;
+  GstCaps *caps;
+  g_return_val_if_fail (str != NULL, NULL);  
+ 
+  s = g_strdup (str);
+  caps = gst_caps_from_string_func (s);
+  g_free (s);
+
+  return caps;
+}
 void
 _gst_caps_initialize (void)
 {
@@ -74,10 +179,12 @@ _gst_caps_initialize (void)
                                        (GBoxedCopyFunc) gst_caps_ref,
                                        (GBoxedFreeFunc) gst_caps_unref);
 
-  g_value_register_transform_func (_gst_caps_type,
-		                   G_TYPE_STRING,
-				   transform_func);
+  g_value_register_transform_func (_gst_caps_type, G_TYPE_STRING,
+				   gst_caps_transform_to_string);
 
+#ifndef GST_DISABLE_TRACE
+  _gst_caps_trace = gst_alloc_trace_register (GST_CAPS_TRACE_NAME);
+#endif
 }
 
 GType
@@ -102,8 +209,9 @@ get_type_for_mime (const gchar *mime)
      definition.typefindfunc = NULL;
 
      factory = gst_type_factory_new (&definition);
-
      typeid = gst_type_register (factory);
+
+     g_free (definition.mime);
   }
   return typeid;
 }
@@ -142,8 +250,11 @@ gst_caps_new_id (const gchar *name, const guint16 id, GstProps *props)
   GstCaps *caps;
 
   caps = gst_mem_chunk_alloc (_gst_caps_chunk);
+#ifndef GST_DISABLE_TRACE
+  gst_alloc_trace_new (_gst_caps_trace, caps);
+#endif
 
-  GST_DEBUG (GST_CAT_CAPS, "new %p", caps);
+  GST_CAT_LOG (GST_CAT_CAPS, "new %p, props %p", caps, props);
 
   gst_props_ref (props);
   gst_props_sink (props);
@@ -155,14 +266,34 @@ gst_caps_new_id (const gchar *name, const guint16 id, GstProps *props)
   caps->refcount = 1;
   GST_CAPS_FLAG_SET (caps, GST_CAPS_FLOATING);
 
-  if (props && !GST_PROPS_IS_FIXED (props))
-    GST_CAPS_FLAG_UNSET (caps, GST_CAPS_FIXED);
-  else
-    GST_CAPS_FLAG_SET (caps, GST_CAPS_FIXED);
-
   return caps;
 }
 
+/**
+ * gst_caps_get_any:
+ *
+ * Return a copy of the caps that represents any capability.
+ *
+ * Returns: the ANY capability
+ */
+GstCaps*
+gst_caps_get_any (void)
+{
+#if 0
+  static GstCaps *caps;
+
+  if (!caps){
+    caps = GST_CAPS_NEW ("gst_caps_any", "*", NULL);
+    gst_caps_ref(caps);
+    gst_caps_sink(caps);
+  }
+
+  return gst_caps_ref(caps);
+#else
+  return NULL;
+#endif
+}
+
 /**
  * gst_caps_replace:
  * @oldcaps: the caps to take replace
@@ -203,7 +334,7 @@ gst_caps_replace_sink (GstCaps **oldcaps, GstCaps *newcaps)
  * Frees the memory used by this caps structure and all
  * the chained caps and properties.
  */
-void
+static void
 gst_caps_destroy (GstCaps *caps)
 {
   GstCaps *next;
@@ -213,11 +344,18 @@ gst_caps_destroy (GstCaps *caps)
 
   next = caps->next;
 
-  GST_DEBUG (GST_CAT_CAPS, "destroy %p", caps);
+  GST_CAT_LOG (GST_CAT_CAPS, "destroy %p", caps);
 
   gst_props_unref (caps->properties);
   g_free (caps->name);
 
+#ifdef USE_POISONING
+  memset(caps, 0xff, sizeof(*caps));
+#endif
+
+#ifndef GST_DISABLE_TRACE
+  gst_alloc_trace_free (_gst_caps_trace, caps);
+#endif
   gst_mem_chunk_free (_gst_caps_chunk, caps);
 
   if (next) 
@@ -234,9 +372,14 @@ gst_caps_destroy (GstCaps *caps)
 void
 gst_caps_debug (GstCaps *caps, const gchar *label)
 {
-  GST_DEBUG_ENTER ("caps debug: %s", label);
+  GST_CAT_DEBUG (GST_CAT_CAPS, "starting caps debug: %s", label);
+  if (caps && caps->refcount == 0) {
+    g_warning ("Warning: refcount of caps %s is 0", label);
+    return;
+  }
+
   while (caps) {
-    GST_DEBUG (GST_CAT_CAPS, "caps: %p %s %s (%sfixed) (refcount %d) %s", 
+    GST_CAT_DEBUG (GST_CAT_CAPS, "caps: %p %s %s (%sfixed) (refcount %d) %s", 
 	       caps, caps->name, gst_caps_get_mime (caps), 
                GST_CAPS_IS_FIXED (caps) ? "" : "NOT ", caps->refcount,
                GST_CAPS_IS_FLOATING (caps) ? "FLOATING" : "");
@@ -245,12 +388,12 @@ gst_caps_debug (GstCaps *caps, const gchar *label)
       gst_props_debug (caps->properties);
     }
     else {
-      GST_DEBUG (GST_CAT_CAPS, "no properties");
+      GST_CAT_DEBUG (GST_CAT_CAPS, "no properties");
     }
 
     caps = caps->next;
   }
-  GST_DEBUG_LEAVE ("caps debug");
+  GST_CAT_DEBUG (GST_CAT_CAPS, "finished caps debug");
 }
 
 /**
@@ -272,7 +415,7 @@ gst_caps_unref (GstCaps *caps)
 
   g_return_val_if_fail (caps->refcount > 0, NULL);
 
-  GST_DEBUG (GST_CAT_CAPS, "unref %p (%d->%d) %d", 
+  GST_CAT_LOG (GST_CAT_CAPS, "unref %p (%d->%d) %d", 
 	     caps, caps->refcount, caps->refcount-1, GST_CAPS_FLAGS (caps));
 
   caps->refcount--;
@@ -301,7 +444,7 @@ gst_caps_ref (GstCaps *caps)
 
   g_return_val_if_fail (caps->refcount > 0, NULL);
 
-  GST_DEBUG (GST_CAT_CAPS, "ref %p (%d->%d) %d", 
+  GST_CAT_LOG (GST_CAT_CAPS, "ref %p (%d->%d) %d", 
 	     caps, caps->refcount, caps->refcount+1, GST_CAPS_FLAGS (caps));
 
   caps->refcount++;
@@ -322,7 +465,7 @@ gst_caps_sink (GstCaps *caps)
     return;
 
   if (GST_CAPS_IS_FLOATING (caps)) {
-    GST_DEBUG (GST_CAT_CAPS, "sink %p", caps);
+    GST_CAT_LOG (GST_CAT_CAPS, "sink %p", caps);
 
     GST_CAPS_FLAG_UNSET (caps, GST_CAPS_FLOATING);
     gst_caps_unref (caps);
@@ -528,11 +671,6 @@ gst_caps_set_props (GstCaps *caps, GstProps *props)
 
   gst_props_replace_sink (&caps->properties, props);
 
-  if (props && !GST_PROPS_IS_FIXED (props))
-    GST_CAPS_FLAG_UNSET (caps, GST_CAPS_FIXED);
-  else
-    GST_CAPS_FLAG_SET (caps, GST_CAPS_FIXED);
-
   return caps;
 }
 
@@ -552,6 +690,66 @@ gst_caps_get_props (GstCaps *caps)
   return caps->properties;
 }
 
+/**
+ * gst_caps_has_property:
+ * @caps: the caps to query
+ * @name: the name of the property to search for
+ *
+ * Figure out whether this caps contains the requested property.
+ *
+ * Returns: true if the caps contains the property.
+ */
+
+gboolean
+gst_caps_has_property (GstCaps *caps, const gchar *name)
+{
+  GstProps *props = gst_caps_get_props (caps);
+
+  return (props != NULL &&
+	  gst_props_has_property (props, name));
+}
+
+/**
+ * gst_caps_has_property_typed:
+ * @caps: the caps to query
+ * @name: the name of the property to search for
+ * @type: the type of the property to search for
+ *
+ * Figure out whether this caps contains the requested property,
+ * and whether this property is of the requested type.
+ *
+ * Returns: true if the caps contains the typed property.
+ */
+
+gboolean
+gst_caps_has_property_typed (GstCaps *caps, const gchar *name, GstPropsType type)
+{
+  GstProps *props = gst_caps_get_props (caps);
+
+  return (props != NULL &&
+	  gst_props_has_property_typed (props, name, type));
+}
+
+/**
+ * gst_caps_has_fixed_property
+ * @caps: the caps to query
+ * @name: the name of the property to search for
+ *
+ * Figure out whether this caps contains the requested property,
+ * and whether this property is fixed.
+ *
+ * Returns: true if the caps contains the fixed property.
+ */
+
+gboolean
+gst_caps_has_fixed_property (GstCaps *caps, const gchar *name)
+{
+  GstProps *props = gst_caps_get_props (caps);
+
+  return (props != NULL &&
+	  gst_props_has_fixed_property (props, name));
+}
+
 /**
  * gst_caps_next:
  * @caps: the caps to query
@@ -681,7 +879,7 @@ static gboolean
 gst_caps_check_compatibility_func (GstCaps *fromcaps, GstCaps *tocaps)
 {
   if (fromcaps->id != tocaps->id) {
-    GST_DEBUG (GST_CAT_CAPS,"mime types differ (%s to %s)",
+    GST_CAT_DEBUG (GST_CAT_CAPS,"mime types differ (%s to %s)",
 	       gst_type_find_by_id (fromcaps->id)->mime, 
 	       gst_type_find_by_id (tocaps->id)->mime);
     return FALSE;
@@ -692,13 +890,13 @@ gst_caps_check_compatibility_func (GstCaps *fromcaps, GstCaps *tocaps)
       return gst_props_check_compatibility (fromcaps->properties, tocaps->properties);
     }
     else {
-      GST_DEBUG (GST_CAT_CAPS,"no source caps");
+      GST_CAT_DEBUG (GST_CAT_CAPS,"no source caps");
       return FALSE;
     }
   }
   else {
     /* assume it accepts everything */
-    GST_DEBUG (GST_CAT_CAPS,"no caps");
+    GST_CAT_DEBUG (GST_CAT_CAPS,"no caps");
     return TRUE;
   }
 }
@@ -719,13 +917,13 @@ gst_caps_is_always_compatible (GstCaps *fromcaps, GstCaps *tocaps)
   if (fromcaps == NULL) {
     if (tocaps == NULL) {
       /* if both are NULL, they can always link.  Think filesrc ! filesink */
-      GST_DEBUG (GST_CAT_CAPS, "both caps NULL, compatible");
+      GST_CAT_DEBUG (GST_CAT_CAPS, "both caps NULL, compatible");
       return TRUE;
     }
     else {
       /* if source caps are NULL, it could be sending anything, so the
        * destination can't know if it can accept this.  Think filesrc ! mad */
-      GST_DEBUG (GST_CAT_CAPS, "source caps NULL, not guaranteed compatible");
+      GST_CAT_DEBUG (GST_CAT_CAPS, "source caps NULL, not guaranteed compatible");
       return FALSE;
     }
   }
@@ -733,7 +931,7 @@ gst_caps_is_always_compatible (GstCaps *fromcaps, GstCaps *tocaps)
     if (tocaps == NULL) {
       /* if the dest caps are NULL, the element can accept anything, always,
        * so they're compatible by definition.  Think mad ! filesink */
-      GST_DEBUG (GST_CAT_CAPS,"destination caps NULL");
+      GST_CAT_DEBUG (GST_CAT_CAPS,"destination caps NULL");
       return TRUE;
     }
   }
@@ -764,7 +962,7 @@ gst_caps_intersect_func (GstCaps *caps1, GstCaps *caps2)
   GstProps *props;
 
   if (caps1->id != caps2->id) {
-    GST_DEBUG (GST_CAT_CAPS, "mime types differ (%s to %s)",
+    GST_CAT_DEBUG (GST_CAT_CAPS, "mime types differ (%s to %s)",
 	       gst_type_find_by_id (caps1->id)->mime, 
 	       gst_type_find_by_id (caps2->id)->mime);
     return NULL;
@@ -803,14 +1001,14 @@ gst_caps_intersect (GstCaps *caps1, GstCaps *caps2)
   GstCaps *result = NULL, *walk = NULL;
 
   /* printing the name is not useful here since caps can be chained */
-  GST_DEBUG (GST_CAT_CAPS, "intersecting caps %p and %p", caps1, caps2);
+  GST_CAT_DEBUG (GST_CAT_CAPS, "intersecting caps %p and %p", caps1, caps2);
 		  
   if (caps1 == NULL) {
-    GST_DEBUG (GST_CAT_CAPS, "first caps is NULL, return other caps");
+    GST_CAT_DEBUG (GST_CAT_CAPS, "first caps is NULL, return other caps");
     return gst_caps_ref (caps2);
   }
   if (caps2 == NULL) {
-    GST_DEBUG (GST_CAT_CAPS, "second caps is NULL, return other caps");
+    GST_CAT_DEBUG (GST_CAT_CAPS, "second caps is NULL, return other caps");
     return gst_caps_ref (caps1);
   }
 
@@ -849,14 +1047,14 @@ gst_caps_union (GstCaps *caps1, GstCaps *caps2)
   GstCaps *result = NULL;
 
   /* printing the name is not useful here since caps can be chained */
-  GST_DEBUG (GST_CAT_CAPS, "making union of caps %p and %p", caps1, caps2);
+  GST_CAT_DEBUG (GST_CAT_CAPS, "making union of caps %p and %p", caps1, caps2);
 		  
   if (caps1 == NULL) {
-    GST_DEBUG (GST_CAT_CAPS, "first caps is NULL, return other caps");
+    GST_CAT_DEBUG (GST_CAT_CAPS, "first caps is NULL, return other caps");
     return gst_caps_ref (caps2);
   }
   if (caps2 == NULL) {
-    GST_DEBUG (GST_CAT_CAPS, "second caps is NULL, return other caps");
+    GST_CAT_DEBUG (GST_CAT_CAPS, "second caps is NULL, return other caps");
     return gst_caps_ref (caps1);
   }
 
@@ -882,7 +1080,7 @@ gst_caps_normalize (GstCaps *caps)
   if (caps == NULL)
     return caps;
 
-  GST_DEBUG (GST_CAT_CAPS, "normalizing caps %p ", caps);
+  GST_CAT_DEBUG (GST_CAT_CAPS, "normalizing caps %p ", caps);
 
   walk = caps;
 
@@ -961,9 +1159,11 @@ gst_caps_load_thyself (xmlNodePtr parent)
       xmlNodePtr subfield = field->xmlChildrenNode;
       GstCaps *caps;
       gchar *content;
-      GstCapsFlags fixed = GST_CAPS_FIXED;
 
       caps = gst_mem_chunk_alloc0 (_gst_caps_chunk);
+#ifndef GST_DISABLE_TRACE
+      gst_alloc_trace_new (_gst_caps_trace, caps);
+#endif
 
       caps->refcount = 1;
       GST_CAPS_FLAG_SET (caps, GST_CAPS_FLOATING);
@@ -984,13 +1184,10 @@ gst_caps_load_thyself (xmlNodePtr parent)
 	  gst_props_ref (props);
 	  gst_props_sink (props);
           caps->properties = props;
-
-          fixed &= (GST_PROPS_IS_FIXED (caps->properties) ? GST_CAPS_FIXED : 0 );
         }
 	
         subfield = subfield->next;
       }
-      GST_CAPS_FLAG_SET (caps, fixed);
 
       result = gst_caps_append (result, caps);
     }
