@@ -28,29 +28,33 @@
 #include <gst/gstdata.h>
 #include <gst/gstformat.h>
 #include <gst/gstobject.h>
+#include <gst/gstcaps.h>
 
 G_BEGIN_DECLS
 
 typedef enum {
-  GST_EVENT_UNKNOWN,
-  GST_EVENT_EOS,
-  GST_EVENT_FLUSH,
-  GST_EVENT_EMPTY,
-  GST_EVENT_DISCONTINUOUS,
-  GST_EVENT_NEW_MEDIA,
-  GST_EVENT_QOS,
-  GST_EVENT_SEEK,
-  GST_EVENT_SEEK_SEGMENT,
-  GST_EVENT_SEGMENT_DONE,
-  GST_EVENT_SIZE,
-  GST_EVENT_RATE,
-  GST_EVENT_FILLER,
-  GST_EVENT_TS_OFFSET,
-  GST_EVENT_INTERRUPT
+  GST_EVENT_UNKNOWN		= 0,
+  GST_EVENT_EOS			= 1,
+  GST_EVENT_FLUSH		= 2,
+  GST_EVENT_EMPTY		= 3,
+  GST_EVENT_DISCONTINUOUS	= 4,
+  /*GST_EVENT_NEW_MEDIA		= 5, <- removed */
+  GST_EVENT_QOS			= 6,
+  GST_EVENT_SEEK		= 7,
+  GST_EVENT_SEEK_SEGMENT	= 8,
+  GST_EVENT_SEGMENT_DONE	= 9,
+  GST_EVENT_SIZE		= 10,
+  GST_EVENT_RATE		= 11,
+  GST_EVENT_FILLER		= 12,
+  GST_EVENT_TS_OFFSET		= 13,
+  GST_EVENT_INTERRUPT		= 14,
+  GST_EVENT_NAVIGATION		= 15
 } GstEventType;
 
 extern GType _gst_event_type;
 
+#define GST_EVENT_TRACE_NAME	"GstEvent"
+
 #define GST_TYPE_EVENT		(_gst_event_type)
 #define GST_EVENT(event)	((GstEvent*)(event))
 #define GST_IS_EVENT(event)	(GST_DATA_TYPE(event) == GST_TYPE_EVENT)
@@ -176,18 +180,22 @@ struct _GstEvent {
     struct {
       gdouble      	value;
     } rate;
+    struct {
+      GstCaps		*caps;
+    } caps;
   } event_data;
+
+  GST_STRUCT_PADDING
 };
 
 void 		_gst_event_initialize 		(void);
-
-void		gst_event_print_stats		(void);
 	
+GType		gst_event_get_type		(void);
 GstEvent*	gst_event_new	        	(GstEventType type);
 
 /* refcounting */
-#define         gst_event_ref(ev)             	gst_data_ref (GST_DATA (ev))
-#define         gst_event_ref_by_count(ev,c)  	gst_data_ref_by_count (GST_DATA (ev), c)
+#define         gst_event_ref(ev)             	GST_EVENT (gst_data_ref (GST_DATA (ev)))
+#define         gst_event_ref_by_count(ev,c)  	GST_EVENT (gst_data_ref_by_count (GST_DATA (ev), c))
 #define         gst_event_unref(ev)           	gst_data_unref (GST_DATA (ev))
 /* copy buffer */
 #define         gst_event_copy(ev)         	GST_EVENT (gst_data_copy (GST_DATA (ev)))
@@ -206,6 +214,9 @@ GstEvent*	gst_event_new_size		(GstFormat format, gint64 value);
 /* discontinous event */
 GstEvent*	gst_event_new_discontinuous	(gboolean new_media,
 						 GstFormat format1, ...);
+GstEvent*	gst_event_new_discontinuousv	(gboolean new_media,
+						 GstFormat format1, 
+						 va_list var_args);
 gboolean	gst_event_discont_get_value	(GstEvent *event, GstFormat format, gint64 *value);
 
 #define		gst_event_new_filler()		gst_event_new(GST_EVENT_FILLER)
