@@ -23,7 +23,11 @@
 #include "gst_private.h"
 
 #include "gstobject.h"
-#include "gstlog.h"
+#include "gstinfo.h"
+
+#ifndef GST_DISABLE_TRACE
+#include "gsttrace.h"
+#endif
 
 /* Object signals and args */
 enum {
@@ -58,10 +62,16 @@ static GType		gst_signal_object_get_type	(void);
 static void		gst_signal_object_class_init	(GstSignalObjectClass *klass);
 static void		gst_signal_object_init		(GstSignalObject *object);
 
+#ifndef GST_DISABLE_LOADSAVE_REGISTRY
 static guint gst_signal_object_signals[SO_LAST_SIGNAL] = { 0 };
+#endif
 
 static void		gst_object_class_init		(GstObjectClass *klass);
 static void		gst_object_init			(GstObject *object);
+#ifndef GST_DISABLE_TRACE
+static GObject *	gst_object_constructor 		(GType type, guint n_construct_properties, 
+							 GObjectConstructParam *construct_params);
+#endif
 
 static void 		gst_object_set_property 	(GObject * object, guint prop_id, const GValue * value,
 		                                    	 GParamSpec * pspec);
@@ -148,12 +158,15 @@ gst_object_class_init (GstObjectClass *klass)
 
   klass->signal_object = g_object_new (gst_signal_object_get_type (), NULL);
 
-  /* see the comments at gst_element_dispatch_properties_changed */
+  /* see the comments at gst_object_dispatch_properties_changed */
   gobject_class->dispatch_properties_changed
 	        = GST_DEBUG_FUNCPTR (gst_object_dispatch_properties_changed);
 
   gobject_class->dispose = gst_object_dispose;
   gobject_class->finalize = gst_object_finalize;
+#ifndef GST_DISABLE_TRACE
+  gobject_class->constructor = gst_object_constructor;
+#endif
 }
 
 static void
@@ -167,6 +180,25 @@ gst_object_init (GstObject *object)
   GST_FLAG_SET (object, GST_FLOATING);
 }
 
+#ifndef GST_DISABLE_TRACE
+static GObject *
+gst_object_constructor (GType type, guint n_construct_properties, GObjectConstructParam *construct_params)
+{
+  const gchar *name;
+  GstAllocTrace *trace;
+  GObject *obj = G_OBJECT_CLASS (parent_class)->constructor (type, n_construct_properties, construct_params);  
+
+  name = g_type_name (type);
+
+  trace = gst_alloc_trace_get (name);
+  if (!trace) {
+    trace = gst_alloc_trace_register (name);
+  }
+  gst_alloc_trace_new (trace, obj);
+  
+  return obj;
+}
+#endif
 /**
  * gst_object_ref:
  * @object: GstObject to reference
@@ -180,8 +212,7 @@ gst_object_ref (GstObject *object)
 {
   g_return_val_if_fail (GST_IS_OBJECT (object), NULL);
 
-  GST_DEBUG (GST_CAT_REFCOUNTING, "ref %p '%s' %d->%d", object,
-	     GST_OBJECT_NAME (object),
+  GST_CAT_LOG_OBJECT (GST_CAT_REFCOUNTING, object, "ref %d->%d",
              G_OBJECT (object)->ref_count,
 	     G_OBJECT (object)->ref_count + 1);
 
@@ -202,8 +233,7 @@ gst_object_unref (GstObject *object)
   g_return_if_fail (GST_IS_OBJECT (object));
   g_return_if_fail (G_OBJECT (object)->ref_count > 0);
 
-  GST_DEBUG (GST_CAT_REFCOUNTING, "unref %p '%s' %d->%d", object,
-	     GST_OBJECT_NAME (object),
+  GST_CAT_LOG_OBJECT (GST_CAT_REFCOUNTING, object, "unref %d->%d",
              G_OBJECT (object)->ref_count,
 	     G_OBJECT (object)->ref_count - 1);
 
@@ -225,7 +255,7 @@ gst_object_sink (GstObject *object)
   g_return_if_fail (object != NULL);
   g_return_if_fail (GST_IS_OBJECT (object));
 
-  GST_DEBUG (GST_CAT_REFCOUNTING, "sink %p '%s'", object, GST_OBJECT_NAME (object));
+  GST_CAT_LOG_OBJECT (GST_CAT_REFCOUNTING, object, "sink");
 
   if (GST_OBJECT_FLOATING (object)) {
     GST_FLAG_UNSET (object, GST_FLOATING);
@@ -234,7 +264,7 @@ gst_object_sink (GstObject *object)
 }
 
 /**
- * gst_object_swap:
+ * gst_object_replace:
  * @oldobj: pointer to place of old GstObject
  * @newobj: new GstObject
  *
@@ -242,46 +272,28 @@ gst_object_sink (GstObject *object)
  * puts the newobj in *oldobj.
  */
 void
-gst_object_swap (GstObject **oldobj, GstObject *newobj)
+gst_object_replace (GstObject **oldobj, GstObject *newobj)
 {
+  g_return_if_fail (oldobj != NULL);
+  g_return_if_fail (*oldobj == NULL || GST_IS_OBJECT (*oldobj));
+  g_return_if_fail (newobj == NULL || GST_IS_OBJECT (newobj));
+	
+  GST_CAT_LOG (GST_CAT_REFCOUNTING, "replace %s %s", 
+	       *oldobj ? GST_STR_NULL (GST_OBJECT_NAME (*oldobj)) : "(NONE)", 
+	       newobj ? GST_STR_NULL (GST_OBJECT_NAME (newobj)) : "(NONE)");
+
   if (*oldobj != newobj) {
-    if (newobj)
-      gst_object_ref (newobj);
-    if (*oldobj)
-      gst_object_unref (*oldobj);
+    if (newobj)  gst_object_ref (newobj);
+    if (*oldobj) gst_object_unref (*oldobj);
 
     *oldobj = newobj;
   }
 }
 
-/**
- * gst_object_destroy:
- * @object: GstObject to destroy
- *
- * Destroy the object.
- * 
- */
-void
-gst_object_destroy (GstObject *object)
-{
-  g_return_if_fail (object != NULL);
-  g_return_if_fail (GST_IS_OBJECT (object));
-
-  GST_DEBUG (GST_CAT_REFCOUNTING, "destroy %p '%s'", object, GST_OBJECT_NAME (object));
-
-  if (!GST_OBJECT_DESTROYED (object))
-  {
-    /* need to hold a reference count around all class method
-     * invocations.
-     */
-    g_object_run_dispose (G_OBJECT (object));
-  }
-}
-
 static void
 gst_object_dispose (GObject *object)
 {
-  GST_DEBUG (GST_CAT_REFCOUNTING, "dispose %p '%s'", object, GST_OBJECT_NAME (object));
+  GST_CAT_LOG_OBJECT (GST_CAT_REFCOUNTING, object, "dispose");
   
   GST_FLAG_SET (GST_OBJECT (object), GST_DESTROYED);
   GST_OBJECT_PARENT (object) = NULL;
@@ -295,21 +307,32 @@ gst_object_finalize (GObject *object)
 {
   GstObject *gstobject = GST_OBJECT (object);
 
-  GST_DEBUG (GST_CAT_REFCOUNTING, "finalize %p '%s'", object, GST_OBJECT_NAME (object));
+  GST_CAT_LOG_OBJECT (GST_CAT_REFCOUNTING, object, "finalize");
 
   g_signal_handlers_destroy (object);
 
-  if (gstobject->name != NULL)
-    g_free (gstobject->name);
+  g_free (gstobject->name);
 
   g_mutex_free (gstobject->lock);
 
+#ifndef GST_DISABLE_TRACE
+  {
+    const gchar *name;
+    GstAllocTrace *trace;
+  
+    name = g_type_name (G_OBJECT_TYPE (object));
+    trace = gst_alloc_trace_get (name);
+    g_assert (trace);
+    gst_alloc_trace_free (trace, object);
+  }
+#endif
+
   parent_class->finalize (object);
 }
 
-/* Changing a GObject property of an element will result in "deep_notify"
- * signals being emitted by the element itself, as well as in each parent
- * element. This is so that an application can connect a listener to the
+/* Changing a GObject property of a GstObject will result in "deep_notify"
+ * signals being emitted by the object itself, as well as in each parent
+ * object. This is so that an application can connect a listener to the
  * top-level bin to catch property-change notifications for all contained
  * elements. */
 static void
@@ -328,7 +351,7 @@ gst_object_dispatch_properties_changed (GObject     *object,
   while (gst_object) {
     /* need own category? */
     for (i = 0; i < n_pspecs; i++) {
-      GST_DEBUG (GST_CAT_EVENT, "deep notification from %s to %s (%s)", GST_OBJECT_NAME (object),
+      GST_CAT_LOG (GST_CAT_EVENT, "deep notification from %s to %s (%s)", GST_OBJECT_NAME (object),
                  GST_OBJECT_NAME (gst_object), pspecs[i]->name);
       g_signal_emit (gst_object, gst_object_signals[DEEP_NOTIFY], g_quark_from_string (pspecs[i]->name),
                      (GstObject *) object, pspecs[i]);
@@ -483,11 +506,7 @@ gst_object_set_parent (GstObject *object, GstObject *parent)
   g_return_if_fail (parent != NULL);
   g_return_if_fail (GST_IS_OBJECT (parent));
   g_return_if_fail (object != parent);
-
-  if (object->parent != NULL) {
-    GST_ERROR_OBJECT (object,object->parent, "object's parent is already set, must unparent first");
-    return;
-  }
+  g_return_if_fail (object->parent == NULL);
 
   gst_object_ref (object);
   gst_object_sink (object);
@@ -527,7 +546,7 @@ gst_object_unparent (GstObject *object)
   if (object->parent == NULL)
     return;
 
-  GST_DEBUG (GST_CAT_REFCOUNTING, "unparent '%s'",GST_OBJECT_NAME(object));
+  GST_CAT_LOG_OBJECT (GST_CAT_REFCOUNTING, object, "unparent");
   
   g_signal_emit (G_OBJECT (object), gst_object_signals[PARENT_UNSET], 0, object->parent);
 
@@ -622,8 +641,7 @@ gst_object_real_restore_thyself (GstObject *object, xmlNodePtr self)
   g_return_if_fail (GST_IS_OBJECT (object));
   g_return_if_fail (self != NULL);
   
-/* FIXME: the signalobject stuff doesn't work
- *  gst_class_signal_emit_by_name (object, "object_loaded", self); */
+   gst_class_signal_emit_by_name (object, "object_loaded", self);
 }
 #endif /* GST_DISABLE_LOADSAVE_REGISTRY */
 
@@ -788,7 +806,7 @@ gst_signal_object_class_init (GstSignalObjectClass *klass)
 #ifndef GST_DISABLE_LOADSAVE_REGISTRY
   gst_signal_object_signals[SO_OBJECT_LOADED] =
     g_signal_new ("object_loaded", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST,
-                  G_STRUCT_OFFSET (GstObjectClass, parent_set), NULL, NULL,
+                  G_STRUCT_OFFSET (GstSignalObjectClass, object_loaded), NULL, NULL,
                   gst_marshal_VOID__OBJECT_POINTER, G_TYPE_NONE, 2,
                   G_TYPE_OBJECT, G_TYPE_POINTER);
 #endif
