@@ -26,17 +26,17 @@
 #include <unistd.h>
 
 #include "gst_private.h"
+
 #include "gstplugin.h"
 #include "gstversion.h"
-#include "gstregistry.h"
-#include "gstlog.h"
+#include "gstregistrypool.h"
+#include "gstinfo.h"
 #include "config.h"
+#include "gstfilter.h"
 
-static GModule *main_module;
+static GModule *main_module = NULL;
+static GList *_gst_plugin_static = NULL;
 
-GList *_gst_plugin_static = NULL;
-
-static void 		gst_plugin_register_statics 	(GModule *module);
 static GstPlugin* 	gst_plugin_register_func 	(GstPluginDesc *desc, GstPlugin *plugin, 
 							 GModule *module);
 GQuark 
@@ -48,42 +48,42 @@ gst_plugin_error_quark (void)
   return quark;
 }
 
-void
-_gst_plugin_initialize (void)
-{
-  main_module =  g_module_open (NULL, G_MODULE_BIND_LAZY);
-  gst_plugin_register_statics (main_module);
-}
-
+/* this function can be called in the GCC constructor extension, before
+ * the _gst_plugin_initialize() was called. In that case, we store the 
+ * plugin description in a list to initialize it when we open the main
+ * module later on.
+ * When the main module is known, we can register the plugin right away.
+ * */
 void
 _gst_plugin_register_static (GstPluginDesc *desc)
 {
-  _gst_plugin_static = g_list_prepend (_gst_plugin_static, desc);
-}
-
-static void
-gst_plugin_register_statics (GModule *module)
-{
-  GList *walk = _gst_plugin_static;
-
-  while (walk) {
-    GstPluginDesc *desc = (GstPluginDesc *) walk->data;
+  if (main_module == NULL) {
+    _gst_plugin_static = g_list_prepend (_gst_plugin_static, desc);
+  }
+  else {
     GstPlugin *plugin;
 
     plugin = g_new0 (GstPlugin, 1);
     plugin->filename = NULL;
     plugin->module = NULL;
-    plugin = gst_plugin_register_func (desc, plugin, module);
+    plugin = gst_plugin_register_func (desc, plugin, main_module);
 
     if (plugin) {
-      plugin->module = module;
+      plugin->module = main_module;
       gst_registry_pool_add_plugin (plugin);
     }
-    
-    walk = g_list_next (walk);
   }
 }
 
+void
+_gst_plugin_initialize (void)
+{
+  main_module =  g_module_open (NULL, G_MODULE_BIND_LAZY);
+
+  /* now register all static plugins */
+  g_list_foreach (_gst_plugin_static, (GFunc) _gst_plugin_register_static, NULL);
+}
+
 static gboolean
 gst_plugin_check_version (gint major, gint minor)
 {
@@ -99,7 +99,7 @@ static GstPlugin*
 gst_plugin_register_func (GstPluginDesc *desc, GstPlugin *plugin, GModule *module)
 {
   if (!gst_plugin_check_version (desc->major_version, desc->minor_version)) {
-    GST_INFO (GST_CAT_PLUGIN_LOADING,"plugin \"%s\" has incompatible version, not loading",
+    GST_CAT_INFO (GST_CAT_PLUGIN_LOADING,"plugin \"%s\" has incompatible version, not loading",
        plugin->filename);
     return NULL;
   }
@@ -108,11 +108,11 @@ gst_plugin_register_func (GstPluginDesc *desc, GstPlugin *plugin, GModule *modul
   plugin->name = g_strdup(desc->name);
 
   if (!((desc->plugin_init) (module, plugin))) {
-    GST_INFO (GST_CAT_PLUGIN_LOADING,"plugin \"%s\" failed to initialise",
+    GST_CAT_INFO (GST_CAT_PLUGIN_LOADING,"plugin \"%s\" failed to initialise",
        plugin->filename);
     return NULL;
   }
-  GST_INFO (GST_CAT_PLUGIN_LOADING,"plugin \"%s\" initialised", plugin->filename);
+  GST_CAT_INFO (GST_CAT_PLUGIN_LOADING,"plugin \"%s\" initialised", GST_STR_NULL (plugin->filename));
 
   return plugin;
 }
@@ -158,7 +158,7 @@ gst_plugin_load_plugin (GstPlugin *plugin, GError **error)
 
   filename = plugin->filename;
 
-  GST_DEBUG (GST_CAT_PLUGIN_LOADING, "attempt to load plugin \"%s\"", filename);
+  GST_CAT_DEBUG (GST_CAT_PLUGIN_LOADING, "attempt to load plugin \"%s\"", filename);
 
   if (g_module_supported () == FALSE) {
     g_set_error (error,
@@ -185,13 +185,13 @@ gst_plugin_load_plugin (GstPlugin *plugin, GError **error)
     if (g_module_symbol (module, "plugin_desc", &ptr)) {
       desc = (GstPluginDesc *)ptr;
 
-      GST_DEBUG (GST_CAT_PLUGIN_LOADING, "plugin \"%s\" loaded, called entry function...", filename);
+      GST_CAT_DEBUG (GST_CAT_PLUGIN_LOADING, "plugin \"%s\" loaded, called entry function...", filename);
 
       plugin->filename = g_strdup (filename);
       plugin = gst_plugin_register_func (desc, plugin, module);
 
       if (plugin != NULL) {
-        GST_INFO (GST_CAT_PLUGIN_LOADING, "plugin \"%s\" loaded", plugin->filename);
+        GST_CAT_INFO (GST_CAT_PLUGIN_LOADING, "plugin \"%s\" loaded", plugin->filename);
         plugin->module = module;
         return TRUE;
       }
@@ -243,11 +243,11 @@ gst_plugin_unload_plugin (GstPlugin *plugin)
 
   if (g_module_close (plugin->module)) {
     plugin->module = NULL;
-    GST_INFO (GST_CAT_PLUGIN_LOADING, "plugin \"%s\" unloaded", plugin->filename);
+    GST_CAT_INFO (GST_CAT_PLUGIN_LOADING, "plugin \"%s\" unloaded", plugin->filename);
     return TRUE;
   }
   else {
-    GST_INFO (GST_CAT_PLUGIN_LOADING, "failed to unload plugin \"%s\"", plugin->filename);
+    GST_CAT_INFO (GST_CAT_PLUGIN_LOADING, "failed to unload plugin \"%s\"", plugin->filename);
     return FALSE;
   }
 }
@@ -280,8 +280,7 @@ gst_plugin_set_name (GstPlugin *plugin, const gchar *name)
 {
   g_return_if_fail (plugin != NULL);
 
-  if (plugin->name)
-    g_free (plugin->name);
+  g_free (plugin->name);
 
   plugin->name = g_strdup (name);
 }
@@ -298,8 +297,7 @@ gst_plugin_set_longname (GstPlugin *plugin, const gchar *longname)
 {
   g_return_if_fail(plugin != NULL);
 
-  if (plugin->longname)
-    g_free(plugin->longname);
+  g_free(plugin->longname);
 
   plugin->longname = g_strdup(longname);
 }
@@ -352,6 +350,100 @@ gst_plugin_is_loaded (GstPlugin *plugin)
   return (plugin->module != NULL);
 }
 
+/**
+ * gst_plugin_feature_list:
+ * @plugin: plugin to query
+ * @filter: the filter to use
+ * @first: only return first match
+ * @user_data: user data passed to the filter function
+ *
+ * Runs a filter against all plugin features and returns a GList with
+ * the results. If the first flag is set, only the first match is 
+ * returned (as a list with a single object).
+ *
+ * Returns: a GList of features, g_list_free after use.
+ */
+GList*
+gst_plugin_feature_filter (GstPlugin *plugin,
+		           GstPluginFeatureFilter filter,
+			   gboolean first,
+			   gpointer user_data)
+{
+  return gst_filter_run (plugin->features, (GstFilterFunc) filter, first, user_data);
+}
+
+typedef struct
+{ 
+  GstPluginFeatureFilter filter;
+  gboolean               first;
+  gpointer               user_data;
+  GList                 *result;
+} FeatureFilterData;
+
+static gboolean
+_feature_filter (GstPlugin *plugin, gpointer user_data)
+{
+  GList *result;
+  FeatureFilterData *data = (FeatureFilterData *) user_data;
+
+  result = gst_plugin_feature_filter (plugin, data->filter, data->first, data->user_data);
+  if (result) {
+    data->result = g_list_concat (data->result, result);
+    return TRUE;
+  }
+  return FALSE;
+}
+
+/**
+ * gst_plugin_list_feature_list:
+ * @list: a list of plugins to query
+ * @filter: the filter to use
+ * @first: only return first match
+ * @user_data: user data passed to the filter function
+ *
+ * Runs a filter against all plugin features of the plugins in the given
+ * list and returns a GList with the results. 
+ * If the first flag is set, only the first match is 
+ * returned (as a list with a single object).
+ *
+ * Returns: a GList of features, g_list_free after use.
+ */
+GList*
+gst_plugin_list_feature_filter  (GList *list, 
+		                 GstPluginFeatureFilter filter,
+			         gboolean first,
+			         gpointer user_data)
+{
+  FeatureFilterData data;
+  GList *result;
+
+  data.filter = filter;
+  data.first = first;
+  data.user_data = user_data;
+  data.result = NULL;
+
+  result = gst_filter_run (list, (GstFilterFunc) _feature_filter, first, &data);
+  g_list_free (result);
+
+  return data.result;
+}
+
+/**
+ * gst_plugin_name_filter:
+ * @plugin: the plugin to check
+ * @name: the name of the plugin
+ *
+ * A standard filterthat returns TRUE when the plugin is of the
+ * given name.
+ *
+ * Returns: TRUE if the plugin is of the given name.
+ */
+gboolean
+gst_plugin_name_filter (GstPlugin *plugin, const gchar *name)
+{
+  return (plugin->name && !strcmp (plugin->name, name));
+}
+
 /**
  * gst_plugin_find_feature:
  * @plugin: plugin to get the feature from
@@ -365,23 +457,23 @@ gst_plugin_is_loaded (GstPlugin *plugin)
 GstPluginFeature*
 gst_plugin_find_feature (GstPlugin *plugin, const gchar *name, GType type)
 {
-  GList *features;
-  
-  g_return_val_if_fail (plugin != NULL, NULL);
+  GList *walk;
+  GstPluginFeature *result = NULL;
+  GstTypeNameData data;
+
   g_return_val_if_fail (name != NULL, NULL);
 
-  features = plugin->features;
+  data.type = type;
+  data.name = name;
+  
+  walk = gst_filter_run (plugin->features, 
+		         (GstFilterFunc) gst_plugin_feature_type_name_filter, TRUE,
+		         &data);
 
-  while (features) {
-    GstPluginFeature *feature = GST_PLUGIN_FEATURE (features->data);
+  if (walk) 
+    result = GST_PLUGIN_FEATURE (walk->data);
 
-    if (!strcmp(GST_PLUGIN_FEATURE_NAME (feature), name) && G_OBJECT_TYPE (feature) == type) {
-      return GST_PLUGIN_FEATURE (feature);
-    }
-
-    features = g_list_next (features);
-  }
-  return NULL;
+  return result;
 }
 
 /**
@@ -418,14 +510,14 @@ gst_plugin_add_feature (GstPlugin *plugin, GstPluginFeature *feature)
  *
  * get a list of all the features that this plugin provides
  *
- * Returns: a GList of features
+ * Returns: a GList of features, use g_list_free to free the list.
  */
 GList*
 gst_plugin_get_feature_list (GstPlugin *plugin)
 {
   g_return_val_if_fail (plugin != NULL, NULL);
 
-  return plugin->features;
+  return g_list_copy (plugin->features);
 }
 
 /**
@@ -446,15 +538,16 @@ gst_plugin_load (const gchar *name)
   if (plugin) {
     gboolean result = gst_plugin_load_plugin (plugin, &error);
     if (error) {
-      GST_DEBUG (GST_CAT_PLUGIN_LOADING, "load_plugin error: %s\n",
+      GST_CAT_DEBUG (GST_CAT_PLUGIN_LOADING, "load_plugin error: %s\n",
 	         error->message);
       g_error_free (error);
     }
     return result;
   }
 
-  GST_DEBUG (GST_CAT_PLUGIN_LOADING, "Could not find %s in registry pool",
+  GST_CAT_DEBUG (GST_CAT_PLUGIN_LOADING, "Could not find %s in registry pool",
              name);
+
   return FALSE;
 }
 
@@ -477,4 +570,3 @@ gst_library_load (const gchar *name)
 
   return res;
 }
-
