@@ -21,10 +21,11 @@
  */
 
 #include "gst_private.h"
+
 #include "gstpluginfeature.h"
 #include "gstplugin.h"
 #include "gstregistry.h"
-#include "gstlog.h"
+#include "gstinfo.h"
 
 static void		gst_plugin_feature_class_init		(GstPluginFeatureClass *klass);
 static void		gst_plugin_feature_init			(GstPluginFeature *feature);
@@ -84,18 +85,24 @@ gst_plugin_feature_init (GstPluginFeature *feature)
 gboolean
 gst_plugin_feature_ensure_loaded (GstPluginFeature *feature)
 {
-  GstPlugin *plugin = (GstPlugin *) (feature->manager);
+  GstPlugin *plugin;
+  
+  g_return_val_if_fail (feature != NULL, FALSE);
+  g_return_val_if_fail (GST_IS_PLUGIN_FEATURE (feature), FALSE);
+
+  plugin = (GstPlugin *) (feature->manager);
 
-  g_assert (feature);
   if (plugin && !gst_plugin_is_loaded (plugin)) {
+#ifndef GST_DISABLE_REGISTRY 
     if (GST_IS_REGISTRY (plugin->manager)) {
-      GST_DEBUG (GST_CAT_PLUGIN_LOADING, 
+      GST_CAT_DEBUG (GST_CAT_PLUGIN_LOADING, 
 	         "loading plugin %s for feature", plugin->name);
 
       if (gst_registry_load_plugin (GST_REGISTRY (plugin->manager), plugin) != GST_REGISTRY_OK)
 	return FALSE;
     }
     else
+#endif /* GST_DISABLE_REGISTRY */
       return FALSE;
   }
   return TRUE;
@@ -122,5 +129,28 @@ gst_plugin_feature_unload_thyself (GstPluginFeature *feature)
     oclass->unload_thyself (feature);
 }
 
+gboolean
+gst_plugin_feature_type_name_filter (GstPluginFeature *feature,
+		                     GstTypeNameData *data)
+{
+  return ((data->type == 0    || data->type == G_OBJECT_TYPE (feature)) &&
+          (data->name == NULL || !strcmp (data->name, GST_PLUGIN_FEATURE_NAME (feature))));
+}
 
+/**
+ * gst_plugin_feature_set_rank:
+ * @feature: feature to rank
+ * @rank: rank value - higher number means more priority rank
+ *
+ * Specifies a rank for a plugin feature, so that autoplugging uses
+ * the most appropriate feature.
+ */
+void
+gst_plugin_feature_set_rank (GstPluginFeature *feature, guint16 rank)
+{
+  g_return_if_fail (feature != NULL);
+  g_return_if_fail (GST_IS_PLUGIN_FEATURE (feature));
+
+  feature->rank = rank;
+}
 
