@@ -1,6 +1,7 @@
 /* GStreamer
  * Copyright (C) 1999,2000 Erik Walthinsen <omega@cse.ogi.edu>
  *                    2000 Wim Taymans <wtay@chello.be>
+ *                    2003 Colin Walters <cwalters@gnome.org>
  *
  * gstqueue.c:
  *
@@ -20,23 +21,13 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/* #define DEBUG_ENABLED */
-/* #define STATUS_ENABLED */
 
-#ifdef STATUS_ENABLED
-#define STATUS(A) GST_DEBUG(GST_CAT_DATAFLOW, A, GST_ELEMENT_NAME(queue))
-#else
-#define STATUS(A)
-#endif
-
-
-#include "config.h"
 #include "gst_private.h"
 
 #include "gstqueue.h"
 #include "gstscheduler.h"
 #include "gstevent.h"
-#include "gstlog.h"
+#include "gstinfo.h"
 
 GstElementDetails gst_queue_details = {
   "Queue",
@@ -51,8 +42,7 @@ GstElementDetails gst_queue_details = {
 
 /* Queue signals and args */
 enum {
-  LOW_WATERMARK,
-  HIGH_WATERMARK,
+  FULL,
   LAST_SIGNAL
 };
 
@@ -67,6 +57,7 @@ enum {
   ARG_LEAKY,
   ARG_LEVEL,
   ARG_MAX_LEVEL,
+  ARG_MIN_THRESHOLD_BYTES, 
   ARG_MAY_DEADLOCK,
   ARG_BLOCK_TIMEOUT,
 };
@@ -81,8 +72,8 @@ static void			gst_queue_set_property		(GObject *object, guint prop_id,
 static void			gst_queue_get_property		(GObject *object, guint prop_id, 
 								 GValue *value, GParamSpec *pspec);
 
-static void			gst_queue_chain			(GstPad *pad, GstBuffer *buf);
-static GstBuffer *		gst_queue_get			(GstPad *pad);
+static void			gst_queue_chain			(GstPad *pad, GstData *data);
+static GstData *		gst_queue_get			(GstPad *pad);
 static GstBufferPool* 		gst_queue_get_bufferpool 	(GstPad *pad);
 	
 static gboolean 		gst_queue_handle_src_event 	(GstPad *pad, GstEvent *event);
@@ -147,12 +138,10 @@ gst_queue_class_init (GstQueueClass *klass)
 
   parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
 
-  gst_queue_signals[HIGH_WATERMARK] =
-    g_signal_new ("high_watermark", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_FIRST,
-                  G_STRUCT_OFFSET (GstQueueClass, high_watermark), NULL, NULL,
-                  g_cclosure_marshal_VOID__INT, G_TYPE_NONE, 1,
-                  G_TYPE_INT);
-
+  gst_queue_signals[FULL] =
+    g_signal_new ("full", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_FIRST,
+                  G_STRUCT_OFFSET (GstQueueClass, full), NULL, NULL,
+		  g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_LEAKY,
     g_param_spec_enum ("leaky", "Leaky", "Where the queue leaks, if at all.",
                        GST_TYPE_QUEUE_LEAKY, GST_QUEUE_NO_LEAK, G_PARAM_READWRITE));
@@ -162,6 +151,10 @@ gst_queue_class_init (GstQueueClass *klass)
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_MAX_LEVEL,
     g_param_spec_int ("max_level", "Maximum Level", "How many buffers the queue holds.",
                       0, G_MAXINT, 100, G_PARAM_READWRITE));
+  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_MIN_THRESHOLD_BYTES,
+    g_param_spec_int ("min_threshold_bytes", "Minimum Threshold",
+                      "Minimum bytes required before signalling not_empty to reader.",
+                      0, G_MAXINT, 0, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_MAY_DEADLOCK,
     g_param_spec_boolean ("may_deadlock", "May Deadlock", "The queue may deadlock if it's full and not PLAYING",
                       TRUE, G_PARAM_READWRITE));
@@ -203,7 +196,7 @@ gst_queue_getcaps (GstPad *pad, GstCaps *caps)
     otherpad = GST_PAD_PEER (queue->sinkpad);
   else
     otherpad = GST_PAD_PEER (queue->srcpad);
-
+  
   if (otherpad)
     return gst_pad_get_caps (otherpad);
 
@@ -235,23 +228,22 @@ gst_queue_init (GstQueue *queue)
   queue->queue = NULL;
   queue->level_buffers = 0;
   queue->level_bytes = 0;
-  queue->level_time = 0LL;
-  queue->size_buffers = 100;		/* 100 buffers */
+  queue->level_time = G_GINT64_CONSTANT (0);
+  queue->size_buffers = 100;				/* 100 buffers */
   queue->size_bytes = 100 * 1024;	/* 100KB */
-  queue->size_time = 1000000000LL;	/* 1sec */
+  queue->size_time = GST_SECOND;	/* 1sec */
+  queue->min_threshold_bytes = 0;
   queue->may_deadlock = TRUE;
   queue->block_timeout = -1;
   queue->interrupt = FALSE;
   queue->flush = FALSE;
 
   queue->qlock = g_mutex_new ();
-  queue->reader = FALSE;
-  queue->writer = FALSE;
   queue->not_empty = g_cond_new ();
   queue->not_full = g_cond_new ();
   queue->events = g_async_queue_new();
   queue->queue = g_queue_new ();
-  GST_DEBUG_ELEMENT (GST_CAT_THREAD, queue, "initialized queue's not_empty & not_full conditions");
+  GST_CAT_DEBUG_OBJECT (GST_CAT_THREAD, queue, "initialized queue's not_empty & not_full conditions");
 }
 
 static void
@@ -264,7 +256,6 @@ gst_queue_dispose (GObject *object)
   g_mutex_free (queue->qlock);
   g_cond_free (queue->not_empty);
   g_cond_free (queue->not_full);
-  gst_queue_locked_flush (queue);
   g_queue_free (queue->queue);
 
   g_async_queue_unref(queue->events);
@@ -285,7 +276,7 @@ gst_queue_get_bufferpool (GstPad *pad)
 static void
 gst_queue_cleanup_data (gpointer data, const gpointer user_data)
 {
-  GST_DEBUG_ELEMENT (GST_CAT_DATAFLOW, user_data, "cleaning buffer %p", data);
+  GST_CAT_DEBUG_OBJECT (GST_CAT_DATAFLOW, user_data, "cleaning buffer %p", data);
 
   gst_data_unref (GST_DATA (data));
 }
@@ -301,20 +292,21 @@ gst_queue_locked_flush (GstQueue *queue)
   queue->timeval = NULL;
   queue->level_buffers = 0;
   queue->level_bytes = 0;
-  queue->level_time = 0LL;
+  queue->level_time = G_GINT64_CONSTANT (0);
   /* make sure any pending buffers to be added are flushed too */
   queue->flush = TRUE;
+  /* signal not_full, since we apparently aren't full anymore */
+  g_cond_signal (queue->not_full);
 }
 
 static void
-gst_queue_chain (GstPad *pad, GstBuffer *buf)
+gst_queue_chain (GstPad *pad, GstData *data)
 {
   GstQueue *queue;
-  gboolean reader;
 
   g_return_if_fail (pad != NULL);
   g_return_if_fail (GST_IS_PAD (pad));
-  g_return_if_fail (buf != NULL);
+  g_return_if_fail (data != NULL);
 
   queue = GST_QUEUE (GST_OBJECT_PARENT (pad));
   
@@ -322,43 +314,44 @@ gst_queue_chain (GstPad *pad, GstBuffer *buf)
   g_async_queue_lock(queue->events);
   while (g_async_queue_length_unlocked(queue->events) > 0){
     GstEvent *event = (GstEvent*)g_async_queue_pop_unlocked(queue->events);
-    GST_DEBUG_ELEMENT (GST_CAT_DATAFLOW, queue, "sending event upstream\n");
+    GST_CAT_DEBUG_OBJECT (GST_CAT_DATAFLOW, queue, "sending event upstream\n");
     gst_pad_event_default (pad, event);
-    GST_DEBUG_ELEMENT (GST_CAT_DATAFLOW, queue, "event sent\n");
+    GST_CAT_DEBUG_OBJECT (GST_CAT_DATAFLOW, queue, "event sent\n");
   }
   g_async_queue_unlock(queue->events);
-
+  
 restart:
   /* we have to lock the queue since we span threads */
-  GST_DEBUG_ELEMENT (GST_CAT_DATAFLOW, queue, "locking t:%p", g_thread_self ());
+  GST_CAT_LOG_OBJECT (GST_CAT_DATAFLOW, queue, "locking t:%p", g_thread_self ());
   g_mutex_lock (queue->qlock);
-  GST_DEBUG_ELEMENT (GST_CAT_DATAFLOW, queue, "locked t:%p", g_thread_self ());
-
+  GST_CAT_LOG_OBJECT (GST_CAT_DATAFLOW, queue, "locked t:%p", g_thread_self ());
+  
   /* assume don't need to flush this buffer when the queue is filled */
   queue->flush = FALSE;
-
-  if (GST_IS_EVENT (buf)) {
-    switch (GST_EVENT_TYPE (buf)) {
+  
+  if (GST_IS_EVENT (data)) {
+    switch (GST_EVENT_TYPE (data)) {
       case GST_EVENT_FLUSH:
-        GST_DEBUG_ELEMENT (GST_CAT_DATAFLOW, queue, "FLUSH event, flushing queue\n");
+        GST_CAT_DEBUG_OBJECT (GST_CAT_DATAFLOW, queue, "FLUSH event, flushing queue\n");
         gst_queue_locked_flush (queue);
 	break;
       case GST_EVENT_EOS:
-	GST_DEBUG_ELEMENT (GST_CAT_DATAFLOW, queue, "eos in on %s %d\n", 
-			   GST_ELEMENT_NAME (queue), queue->level_buffers);
+	GST_CAT_DEBUG_OBJECT (GST_CAT_DATAFLOW, queue, "eos in on %s %d\n", 
+                              GST_ELEMENT_NAME (queue), queue->level_buffers);
 	break;
       default:
 	/* we put the event in the queue, we don't have to act ourselves */
 	break;
     }
   }
-
-  GST_DEBUG_ELEMENT (GST_CAT_DATAFLOW, queue, "adding buffer %p of size %d",buf,GST_BUFFER_SIZE(buf));
+  
+  if (GST_IS_BUFFER (data))
+    GST_CAT_LOG_OBJECT (GST_CAT_DATAFLOW, queue, 
+                        "adding buffer %p of size %d", data, GST_BUFFER_SIZE (data));
 
   if (queue->level_buffers == queue->size_buffers) {
     g_mutex_unlock (queue->qlock);
-    g_signal_emit (G_OBJECT (queue), gst_queue_signals[HIGH_WATERMARK], 0,
-                   queue->level_buffers);
+    g_signal_emit (G_OBJECT (queue), gst_queue_signals[FULL], 0);
     g_mutex_lock (queue->qlock);
 
     /* if this is a leaky queue... */
@@ -366,65 +359,66 @@ restart:
       /* FIXME don't want to leak events! */
       /* if we leak on the upstream side, drop the current buffer */
       if (queue->leaky == GST_QUEUE_LEAK_UPSTREAM) {
-        GST_DEBUG_ELEMENT (GST_CAT_DATAFLOW, queue, "queue is full, leaking buffer on upstream end");
-        if (GST_IS_EVENT (buf))
+        GST_CAT_DEBUG_OBJECT (GST_CAT_DATAFLOW, queue, "queue is full, leaking buffer on upstream end");
+        if (GST_IS_EVENT (data))
           fprintf(stderr, "Error: queue [%s] leaked an event, type:%d\n",
               GST_ELEMENT_NAME(GST_ELEMENT(queue)),
-              GST_EVENT_TYPE(GST_EVENT(buf)));
-          GST_DEBUG_ELEMENT (GST_CAT_DATAFLOW, queue, "queue is full, leaking buffer on upstream end");
-        gst_buffer_unref(buf);
+              GST_EVENT_TYPE(GST_EVENT(data)));
         /* now we have to clean up and exit right away */
         g_mutex_unlock (queue->qlock);
-        return;
+        goto out_unref;
       }
       /* otherwise we have to push a buffer off the other end */
       else {
         gpointer front;
-        GstBuffer *leakbuf;
+        GstData *leak;
 
-        GST_DEBUG_ELEMENT (GST_CAT_DATAFLOW, queue, "queue is full, leaking buffer on downstream end");
+        GST_CAT_DEBUG_OBJECT (GST_CAT_DATAFLOW, queue, "queue is full, leaking buffer on downstream end");
 
         front = g_queue_pop_head (queue->queue);
-        leakbuf = (GstBuffer *)(front);
+        leak = GST_DATA (front);
 
-        if (GST_IS_EVENT (leakbuf)) {
+        queue->level_buffers--;
+        if (GST_IS_EVENT (leak)) {
           fprintf(stderr, "Error: queue [%s] leaked an event, type:%d\n",
               GST_ELEMENT_NAME(GST_ELEMENT(queue)),
-              GST_EVENT_TYPE(GST_EVENT(leakbuf)));
-	}
-        queue->level_buffers--;
-        queue->level_bytes -= GST_BUFFER_SIZE(leakbuf);
-        gst_data_unref (GST_DATA (leakbuf));
+              GST_EVENT_TYPE(GST_EVENT(leak)));
+	} else {
+          queue->level_bytes -= GST_BUFFER_SIZE(leak);
+        }
+
+        gst_data_unref (leak);
       }
     }
 
-    GST_DEBUG_ELEMENT (GST_CAT_DATAFLOW, queue, "pre full wait, level:%d/%d",
-        		queue->level_buffers, queue->size_buffers);
+    GST_CAT_LOG_OBJECT (GST_CAT_DATAFLOW, queue, "pre full wait, level:%d/%d buffers, %d bytes",
+        		queue->level_buffers, queue->size_buffers, queue->level_bytes);
 
     while (queue->level_buffers == queue->size_buffers) {
       /* if there's a pending state change for this queue or its manager, switch */
       /* back to iterator so bottom half of state change executes */
       if (queue->interrupt) {
-        GST_DEBUG_ELEMENT (GST_CAT_DATAFLOW, queue, "interrupted!!");
+        GST_CAT_DEBUG_OBJECT (GST_CAT_DATAFLOW, queue, "interrupted!!");
         g_mutex_unlock (queue->qlock);
 	if (gst_scheduler_interrupt (gst_pad_get_scheduler (queue->sinkpad), GST_ELEMENT (queue)))
-          return;
-	/* if we got here bacause we were unlocked after a flush, we don't need
+          goto out_unref;
+	/* if we got here because we were unlocked after a flush, we don't need
 	 * to add the buffer to the queue again */
 	if (queue->flush) {
-          GST_DEBUG_ELEMENT (GST_CAT_DATAFLOW, queue, "not adding pending buffer after flush");
-	  return;
+          GST_CAT_DEBUG_OBJECT (GST_CAT_DATAFLOW, queue, "not adding pending buffer after flush");
+	  goto out_unref;
 	}
-        GST_DEBUG_ELEMENT (GST_CAT_DATAFLOW, queue, "adding pending buffer after interrupt");
+        GST_CAT_DEBUG_OBJECT (GST_CAT_DATAFLOW, queue, "adding pending buffer after interrupt");
 	goto restart;
       }
       if (GST_STATE (queue) != GST_STATE_PLAYING) {
 	/* this means the other end is shut down */
 	/* try to signal to resolve the error */
 	if (!queue->may_deadlock) {
-          gst_data_unref (GST_DATA (buf));
           g_mutex_unlock (queue->qlock);
+          gst_data_unref (data);
           gst_element_error (GST_ELEMENT (queue), "deadlock found, source pad elements are shut down");
+	  /* we don't want to goto out_unref here, since we want to clean up before calling gst_element_error */
 	  return;
 	}
 	else {
@@ -432,51 +426,46 @@ restart:
 	}
       }
 
-      GST_DEBUG_ELEMENT (GST_CAT_DATAFLOW, queue, "waiting for not_full, level:%d/%d", 
-		      queue->level_buffers, queue->size_buffers);
-      if (queue->writer)
-        GST_DEBUG_ELEMENT (GST_CAT_DATAFLOW, queue, "WARNING: multiple writers on queue!");
-      queue->writer = TRUE;
+      GST_CAT_DEBUG_OBJECT (GST_CAT_DATAFLOW, queue, "waiting for not_full, level:%d/%d buffers, %d bytes", 
+		      queue->level_buffers, queue->size_buffers, queue->level_bytes);
       g_cond_wait (queue->not_full, queue->qlock);
-      queue->writer = FALSE;
-      GST_DEBUG_ELEMENT (GST_CAT_DATAFLOW, queue, "got not_full signal");
+      GST_CAT_DEBUG_OBJECT (GST_CAT_DATAFLOW, queue, "got not_full signal");
     }
-    GST_DEBUG_ELEMENT (GST_CAT_DATAFLOW, queue, "post full wait, level:%d/%d",
-        queue->level_buffers, queue->size_buffers);
+    GST_CAT_LOG_OBJECT (GST_CAT_DATAFLOW, queue, "post full wait, level:%d/%d buffers, %d bytes",
+                        queue->level_buffers, queue->size_buffers, queue->level_bytes);
   }
 
   /* put the buffer on the tail of the list */
-  g_queue_push_tail (queue->queue, buf);
+  g_queue_push_tail (queue->queue, data);
 
   queue->level_buffers++;
-  queue->level_bytes += GST_BUFFER_SIZE(buf);
+  if (GST_IS_BUFFER (data))
+    queue->level_bytes += GST_BUFFER_SIZE (data);
 
   /* this assertion _has_ to hold */
   g_assert (queue->queue->length == queue->level_buffers);
 
-  GST_DEBUG_ELEMENT (GST_CAT_DATAFLOW, queue, "(%s:%s)+ level:%d/%d",
+  GST_CAT_LOG_OBJECT (GST_CAT_DATAFLOW, queue, "(%s:%s)+ level:%d/%d buffers, %d bytes",
       GST_DEBUG_PAD_NAME(pad),
-      queue->level_buffers, queue->size_buffers);
-
-  /* reader waiting on an empty queue */
-  reader = queue->reader;
+      queue->level_buffers, queue->size_buffers, queue->level_bytes);
 
+  GST_CAT_LOG_OBJECT (GST_CAT_DATAFLOW, queue, "signalling not_empty");
+  g_cond_signal (queue->not_empty);
   g_mutex_unlock (queue->qlock);
 
-  if (reader)
-  {
-    GST_DEBUG_ELEMENT (GST_CAT_DATAFLOW, queue, "signalling not_empty");
-    g_cond_signal (queue->not_empty);
-  }
+  return;
+
+out_unref:
+  gst_data_unref (data);
+  return;
 }
 
-static GstBuffer *
+static GstData *
 gst_queue_get (GstPad *pad)
 {
   GstQueue *queue;
-  GstBuffer *buf = NULL;
+  GstData *data = NULL;
   gpointer front;
-  gboolean writer;
 
   g_assert(pad != NULL);
   g_assert(GST_IS_PAD(pad));
@@ -487,21 +476,22 @@ gst_queue_get (GstPad *pad)
 
 restart:
   /* have to lock for thread-safety */
-  GST_DEBUG_ELEMENT (GST_CAT_DATAFLOW, queue, "locking t:%p", g_thread_self ());
+  GST_CAT_LOG_OBJECT (GST_CAT_DATAFLOW, queue, "locking t:%p", g_thread_self ());
   g_mutex_lock (queue->qlock);
-  GST_DEBUG_ELEMENT (GST_CAT_DATAFLOW, queue, "locked t:%p %p", g_thread_self (), queue->not_empty);
+  GST_CAT_LOG_OBJECT (GST_CAT_DATAFLOW, queue, "locked t:%p %p", g_thread_self (), queue->not_empty);
 
-  GST_DEBUG_ELEMENT (GST_CAT_DATAFLOW, queue, "pre empty wait, level:%d/%d", queue->level_buffers, queue->size_buffers);
+  GST_CAT_LOG_OBJECT (GST_CAT_DATAFLOW, queue, "pre empty wait, level:%d/%d buffers, %d bytes",
+    queue->level_buffers, queue->size_buffers, queue->level_bytes);
   while (queue->level_buffers == 0) {
     /* if there's a pending state change for this queue or its manager, switch
      * back to iterator so bottom half of state change executes
      */ 
     //while (GST_STATE_PENDING (queue) != GST_STATE_VOID_PENDING) {
     if (queue->interrupt) {
-      GST_DEBUG_ELEMENT (GST_CAT_DATAFLOW, queue, "interrupted!!");
+      GST_CAT_DEBUG_OBJECT (GST_CAT_DATAFLOW, queue, "interrupted!!");
       g_mutex_unlock (queue->qlock);
       if (gst_scheduler_interrupt (gst_pad_get_scheduler (queue->srcpad), GST_ELEMENT (queue)))
-        return GST_BUFFER (gst_event_new (GST_EVENT_INTERRUPT));
+        return GST_DATA (gst_event_new (GST_EVENT_INTERRUPT));
       goto restart;
     }
     if (GST_STATE (queue) != GST_STATE_PLAYING) {
@@ -516,12 +506,10 @@ restart:
       }
     }
 
-    GST_DEBUG_ELEMENT (GST_CAT_DATAFLOW, queue, "waiting for not_empty, level:%d/%d", queue->level_buffers, queue->size_buffers);
-    if (queue->reader)
-      GST_DEBUG_ELEMENT (GST_CAT_DATAFLOW, queue, "WARNING: multiple readers on queue!");
-    queue->reader = TRUE;
+    GST_CAT_DEBUG_OBJECT (GST_CAT_DATAFLOW, queue, "waiting for not_empty, level:%d/%d buffers, %d bytes",
+                       queue->level_buffers, queue->size_buffers, queue->level_bytes);
     
-    //if (queue->block_timeout > -1){
+    /* if (queue->block_timeout > -1){ */
     if (FALSE) {
       GTimeVal timeout;
       g_get_current_time(&timeout);
@@ -529,48 +517,43 @@ restart:
       if (!g_cond_timed_wait (queue->not_empty, queue->qlock, &timeout)){
         g_mutex_unlock (queue->qlock);
 	g_warning ("filler");
-        return GST_BUFFER(gst_event_new_filler());
+        return GST_DATA (gst_event_new_filler());
       }
     }
     else {
       g_cond_wait (queue->not_empty, queue->qlock);
     }
-    queue->reader = FALSE;
-    GST_DEBUG_ELEMENT (GST_CAT_DATAFLOW, queue, "got not_empty signal");
+    GST_CAT_DEBUG_OBJECT (GST_CAT_DATAFLOW, queue, "got not_empty signal");
   }
-  GST_DEBUG_ELEMENT (GST_CAT_DATAFLOW, queue, "post empty wait, level:%d/%d", queue->level_buffers, queue->size_buffers);
+  GST_CAT_LOG_OBJECT (GST_CAT_DATAFLOW, queue, "post empty wait, level:%d/%d buffers, %d bytes",
+    queue->level_buffers, queue->size_buffers, queue->level_bytes);
 
   front = g_queue_pop_head (queue->queue);
-  buf = (GstBuffer *)(front);
-  GST_DEBUG_ELEMENT (GST_CAT_DATAFLOW, queue, "retrieved buffer %p from queue", buf);
+  data = GST_DATA (front);
+  GST_CAT_LOG_OBJECT (GST_CAT_DATAFLOW, queue, "retrieved data %p from queue", data);
 
   queue->level_buffers--;
-  queue->level_bytes -= GST_BUFFER_SIZE(buf);
+  if (GST_IS_BUFFER (data))
+    queue->level_bytes -= GST_BUFFER_SIZE (data);
 
-  GST_DEBUG_ELEMENT (GST_CAT_DATAFLOW, queue, "(%s:%s)- level:%d/%d",
+  GST_CAT_LOG_OBJECT (GST_CAT_DATAFLOW, queue, "(%s:%s)- level:%d/%d buffers, %d bytes",
       GST_DEBUG_PAD_NAME(pad),
-      queue->level_buffers, queue->size_buffers);
+      queue->level_buffers, queue->size_buffers, queue->level_bytes);
 
   /* this assertion _has_ to hold */
   g_assert (queue->queue->length == queue->level_buffers);
 
-  /* writer waiting on a full queue */
-  writer = queue->writer;
+  GST_CAT_LOG_OBJECT (GST_CAT_DATAFLOW, queue, "signalling not_full");
+  g_cond_signal (queue->not_full);
 
   g_mutex_unlock (queue->qlock);
 
-  if (writer)
-  {
-    GST_DEBUG_ELEMENT (GST_CAT_DATAFLOW, queue, "signalling not_full");
-    g_cond_signal (queue->not_full);
-  }
-
   /* FIXME where should this be? locked? */
-  if (GST_IS_EVENT(buf)) {
-    GstEvent *event = GST_EVENT(buf);
+  if (GST_IS_EVENT (data)) {
+    GstEvent *event = GST_EVENT (data);
     switch (GST_EVENT_TYPE(event)) {
       case GST_EVENT_EOS:
-        GST_DEBUG_ELEMENT (GST_CAT_DATAFLOW, queue, "queue \"%s\" eos", GST_ELEMENT_NAME (queue));
+        GST_CAT_DEBUG_OBJECT (GST_CAT_DATAFLOW, queue, "queue \"%s\" eos", GST_ELEMENT_NAME (queue));
         gst_element_set_eos (GST_ELEMENT (queue));
         break;
       default:
@@ -578,7 +561,7 @@ restart:
     }
   }
 
-  return buf;
+  return data;
 }
 
 
@@ -612,7 +595,7 @@ gst_queue_handle_src_event (GstPad *pad, GstEvent *event)
 
   switch (event_type) {
     case GST_EVENT_FLUSH:
-      GST_DEBUG_ELEMENT (GST_CAT_DATAFLOW, queue, "FLUSH event, flushing queue\n");
+      GST_CAT_DEBUG_OBJECT (GST_CAT_DATAFLOW, queue, "FLUSH event, flushing queue\n");
       gst_queue_locked_flush (queue);
       break;
     case GST_EVENT_SEEK:
@@ -654,7 +637,7 @@ gst_queue_change_state (GstElement *element)
 
   queue = GST_QUEUE (element);
 
-  GST_DEBUG_ENTER("('%s')", GST_ELEMENT_NAME (element));
+  GST_CAT_LOG_OBJECT (GST_CAT_STATES, element, "starting state change");
 
   /* lock the queue so another thread (not in sync with this thread's state)
    * can't call this queue's _get (or whatever)
@@ -669,10 +652,9 @@ gst_queue_change_state (GstElement *element)
       break;
     case GST_STATE_PAUSED_TO_PLAYING:
       if (!GST_PAD_IS_LINKED (queue->sinkpad)) {
-        GST_DEBUG_ELEMENT (GST_CAT_STATES, queue, "queue %s is not linked", GST_ELEMENT_NAME (queue));
+        GST_CAT_DEBUG_OBJECT (GST_CAT_STATES, queue, "queue %s is not linked", GST_ELEMENT_NAME (queue));
         /* FIXME can this be? */
-        if (queue->reader)
-          g_cond_signal (queue->not_empty);
+        g_cond_signal (queue->not_empty);
 
         ret = GST_STATE_FAILURE;
         goto error;
@@ -684,10 +666,10 @@ gst_queue_change_state (GstElement *element)
         sink_sched = gst_pad_get_scheduler (GST_PAD_CAST (queue->sinkpad));
 
         if (src_sched == sink_sched) {
-          GST_DEBUG_ELEMENT (GST_CAT_STATES, queue, "queue %s does not connect different schedulers", 
+          GST_CAT_DEBUG_OBJECT (GST_CAT_STATES, queue, "queue %s does not connect different schedulers", 
 			GST_ELEMENT_NAME (queue));
 
-	  g_warning ("queue %s does connect different schedulers",
+	  g_warning ("queue %s does not connect different schedulers",
 			GST_ELEMENT_NAME (queue));
 
 	  ret = GST_STATE_FAILURE;
@@ -710,7 +692,7 @@ gst_queue_change_state (GstElement *element)
 error:
   g_mutex_unlock (queue->qlock);
 
-  GST_DEBUG_LEAVE("('%s')", GST_ELEMENT_NAME (element));
+  GST_CAT_LOG_OBJECT (GST_CAT_STATES, element, "done with state change");
   return ret;
 }
 
@@ -732,6 +714,9 @@ gst_queue_set_property (GObject *object, guint prop_id, const GValue *value, GPa
     case ARG_MAX_LEVEL:
       queue->size_buffers = g_value_get_int (value);
       break;
+    case ARG_MIN_THRESHOLD_BYTES:
+      queue->min_threshold_bytes = g_value_get_int (value);
+      break;
     case ARG_MAY_DEADLOCK:
       queue->may_deadlock = g_value_get_boolean (value);
       break;
@@ -764,6 +749,9 @@ gst_queue_get_property (GObject *object, guint prop_id, GValue *value, GParamSpe
     case ARG_MAX_LEVEL:
       g_value_set_int (value, queue->size_buffers);
       break;
+    case ARG_MIN_THRESHOLD_BYTES:
+      g_value_set_int (value, queue->min_threshold_bytes);
+      break;
     case ARG_MAY_DEADLOCK:
       g_value_set_boolean (value, queue->may_deadlock);
       break;
