@@ -27,6 +27,7 @@
 #ifndef GST_DISABLE_TYPE_FIND
 
 #include <gst/gstelement.h>
+#include <gst/gstbytestream.h>
 
 G_BEGIN_DECLS
 
@@ -46,18 +47,21 @@ struct _GstTypeFind {
   GstElement 	 element;
 
   GstPad 	*sinkpad;
+  GstByteStream *bs;
 
   GstCaps 	*caps;
 
-  gint 		 num_buffer;
-  gint 		 max_buffers;
+  GST_OBJECT_PADDING
 };
 
 struct _GstTypeFindClass {
   GstElementClass 	parent_class;
 
   /* signals */
-  void 			(*have_type) 	(GstElement *element);
+  void 			(*have_type) 	(GstElement *element,
+					 GstCaps    *caps);
+
+  GST_CLASS_PADDING
 };
 
 GType gst_type_find_get_type (void);
