@@ -20,7 +20,10 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/* #define DEBUG_ENABLED */
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
+
 #include <stdio.h>
 #include <errno.h>
 #include <sys/types.h>
@@ -35,6 +38,7 @@
 #include <gst/gstscheduler.h>
 #include <gst/gstautoplug.h>
 #include <gst/gsturi.h>
+#include <gst/gstinfo.h>
 
 #include "gstxmlregistry.h"
 
@@ -422,7 +426,7 @@ plugin_times_older_than_recurse(gchar *path, time_t regtime)
   time_t pathtime = get_time(path);
 
   if (pathtime > regtime) {
-    GST_INFO (GST_CAT_PLUGIN_LOADING,
+    GST_CAT_INFO (GST_CAT_PLUGIN_LOADING,
                      "time for %s was %ld; more recent than registry time of %ld\n",
                      path, (long)pathtime, (long)regtime);
     return FALSE;
@@ -455,7 +459,7 @@ plugin_times_older_than(GList *paths, time_t regtime)
    */
 
   while (paths) {
-    GST_DEBUG (GST_CAT_PLUGIN_LOADING,
+    GST_CAT_DEBUG (GST_CAT_PLUGIN_LOADING,
                       "comparing plugin times from %s with %ld\n",
                       (gchar *)paths->data, (long) regtime);
     if(!plugin_times_older_than_recurse(paths->data, regtime))
@@ -485,10 +489,10 @@ gst_xml_registry_open_func (GstXMLRegistry *registry, GstXMLRegistryMode mode)
       /* if it's not writable, then don't bother */
       if (!(gst_registry->flags & GST_REGISTRY_WRITABLE))
       {
-        GST_INFO (GST_CAT_GST_INIT, "Registry isn't writable");
+        GST_CAT_INFO (GST_CAT_GST_INIT, "Registry isn't writable");
 	return FALSE;
       }
-      GST_INFO (GST_CAT_GST_INIT, "Registry doesn't exist, trying to build...");
+      GST_CAT_INFO (GST_CAT_GST_INIT, "Registry doesn't exist, trying to build...");
       gst_registry_rebuild (gst_registry);
       gst_registry_save (gst_registry);
       /* FIXME: verify that the flags actually get updated ! */
@@ -502,18 +506,18 @@ gst_xml_registry_open_func (GstXMLRegistry *registry, GstXMLRegistryMode mode)
 
     if (!plugin_times_older_than (paths, get_time (registry->location))) {
       if (gst_registry->flags & GST_REGISTRY_WRITABLE) {
-        GST_INFO (GST_CAT_GST_INIT, "Registry out of date, rebuilding...");
+        GST_CAT_INFO (GST_CAT_GST_INIT, "Registry out of date, rebuilding...");
       
         gst_registry_rebuild (gst_registry);
 
         gst_registry_save (gst_registry);
 
         if (!plugin_times_older_than (paths, get_time (registry->location))) {
-          GST_INFO (GST_CAT_GST_INIT, "Registry still out of date, something is wrong...");
+          GST_CAT_INFO (GST_CAT_GST_INIT, "Registry still out of date, something is wrong...");
           return FALSE;
         }
       } else {
-        GST_INFO (GST_CAT_PLUGIN_LOADING, "Can't write to this registry and it's out of date, ignoring it");
+        GST_CAT_INFO (GST_CAT_PLUGIN_LOADING, "Can't write to this registry and it's out of date, ignoring it");
         return FALSE;
       }
     }
@@ -613,7 +617,7 @@ gst_xml_registry_load (GstRegistry *registry)
   seconds = g_timer_elapsed (timer, NULL);
   g_timer_destroy (timer);
 
-  GST_INFO (0, "registry: loaded %s in %f seconds\n          (%s)", 
+  GST_INFO ( "registry: loaded %s in %f seconds\n          (%s)", 
 	   registry->name, seconds, xmlregistry->location);
 
   CLASS (xmlregistry)->close_func (xmlregistry);
@@ -664,60 +668,49 @@ gst_xml_registry_parse_element_factory (GMarkupParseContext *context, const gcha
   GstElementFactory *factory = GST_ELEMENT_FACTORY (registry->current_feature);
 
   if (!strcmp (tag, "name")) {
+    g_free (registry->current_feature->name);
     registry->current_feature->name = g_strndup (text, text_len);
   }
   else if (!strcmp (tag, "longname")) {
+    g_free (factory->details->longname);
     factory->details->longname = g_strndup (text, text_len);
   }
   else if (!strcmp(tag, "class")) {
+    g_free (factory->details->klass);
     factory->details->klass = g_strndup (text, text_len);
   }
   else if (!strcmp(tag, "description")) {
+    g_free (factory->details->description);
     factory->details->description = g_strndup (text, text_len);
   }
+  else if (!strcmp(tag, "license")) {
+    g_free (factory->details->license);
+    factory->details->license = g_strndup (text, text_len);
+  }
   else if (!strcmp(tag, "version")) {
+    g_free (factory->details->version);
     factory->details->version = g_strndup (text, text_len);
   }
   else if (!strcmp(tag, "author")) {
+    g_free (factory->details->author);
     factory->details->author = g_strndup (text, text_len);
   }
   else if (!strcmp(tag, "copyright")) {
+    g_free (factory->details->copyright);
     factory->details->copyright = g_strndup (text, text_len);
   }
   else if (!strcmp(tag, "rank")) {
-    gint value;
-    sscanf (text, "%d", &value);
-    factory->rank = value;
-  }
-
-  return TRUE;
-}
-
-static GstCaps*
-gst_type_type_find_dummy (GstBuffer *buffer, gpointer priv)
-{
-  GstTypeFactory *factory = (GstTypeFactory *)priv;
-
-  GST_DEBUG (GST_CAT_TYPES,"gsttype: need to load typefind function for %s", factory->mime);
-
-  if (gst_plugin_feature_ensure_loaded (GST_PLUGIN_FEATURE (factory))) {
-    if (factory->typefindfunc)
-    {
-      if (factory->typefindfunc == gst_type_type_find_dummy) {
-        g_warning ("factory->typefindfunc set to _dummy, not recursing, FIXME");
-      }
-      else
-      {
-        GstCaps *res = factory->typefindfunc (buffer, factory);
-        if (res)
-          return res;
-      }
+    gint rank;
+    gchar *ret;
+     rank = strtol (text, &ret, 0);
+    if (ret == text + text_len) {
+     gst_element_factory_set_rank (factory, rank);
     }
   }
-  return NULL;
+  
+  return TRUE;
 }
 
-
 static gboolean
 gst_xml_registry_parse_type_factory (GMarkupParseContext *context, const gchar *tag, const gchar *text,
                                      gsize text_len, GstXMLRegistry *registry, GError **error)
@@ -909,7 +902,6 @@ gst_xml_registry_start_element (GMarkupParseContext *context,
 	    factory->details_dynamic = TRUE;
 	    factory->details = g_new0(GstElementDetails, 1);
 	    factory->padtemplates = NULL;
-	    factory->rank = 0;
 	    xmlregistry->parser = gst_xml_registry_parse_element_factory;
 	    break;
 	  }
@@ -1293,7 +1285,7 @@ gst_xml_registry_save_props_func (GstPropsEntry *entry,
 
   name = gst_props_entry_get_name (entry);
 
-  switch (gst_props_entry_get_type (entry)) {
+  switch (gst_props_entry_get_props_type (entry)) {
     case GST_PROPS_INT_TYPE:
     {
       gint value;
@@ -1326,7 +1318,8 @@ gst_xml_registry_save_props_func (GstPropsEntry *entry,
     {
       guint32 fourcc;
       gst_props_entry_get_fourcc_int (entry, &fourcc);
-      CLASS (xmlregistry)->save_func (xmlregistry, "<!--%4.4s-->\n", (gchar *)&fourcc);
+      CLASS (xmlregistry)->save_func (xmlregistry, "<!-- "GST_FOURCC_FORMAT" -->\n", 
+				      GST_FOURCC_ARGS (fourcc));
       CLASS (xmlregistry)->save_func (xmlregistry, "<fourcc name=\"%s\" hexvalue=\"%08x\"/>\n", name, fourcc);
       break;
     }
@@ -1345,7 +1338,7 @@ gst_xml_registry_save_props_func (GstPropsEntry *entry,
       break;
     }
     default:
-      g_warning ("trying to save unknown property type %d", gst_props_entry_get_type (entry));
+      g_warning ("trying to save unknown property type %d", gst_props_entry_get_props_type (entry));
       return FALSE;
   }
   return TRUE;
@@ -1361,7 +1354,7 @@ gst_xml_registry_save_props (GstXMLRegistry *xmlregistry, GstProps *props)
   while (proplist) {
     GstPropsEntry *entry = (GstPropsEntry *) proplist->data;
 
-    switch (gst_props_entry_get_type (entry)) {
+    switch (gst_props_entry_get_props_type (entry)) {
       case GST_PROPS_LIST_TYPE: 
       {
 	const GList *list;
@@ -1438,6 +1431,11 @@ gst_xml_registry_save_feature (GstXMLRegistry *xmlregistry, GstPluginFeature *fe
 {
   PUT_ESCAPED ("name", feature->name);
 
+  if (feature->rank > 0) {
+    gint rank = feature->rank;
+    CLASS (xmlregistry)->save_func (xmlregistry, "<rank>%d</rank>\n", rank);
+  }	
+    
   if (GST_IS_ELEMENT_FACTORY (feature)) {
     GstElementFactory *factory = GST_ELEMENT_FACTORY (feature);
     GList *templates;
@@ -1445,16 +1443,11 @@ gst_xml_registry_save_feature (GstXMLRegistry *xmlregistry, GstPluginFeature *fe
     PUT_ESCAPED ("longname", factory->details->longname);
     PUT_ESCAPED ("class", factory->details->klass);
     PUT_ESCAPED ("description", factory->details->description);
+    PUT_ESCAPED ("license", factory->details->license);
     PUT_ESCAPED ("version", factory->details->version);
     PUT_ESCAPED ("author", factory->details->author);
     PUT_ESCAPED ("copyright", factory->details->copyright);
     
-    /* only write the rank if it is greater than zero */
-    if (factory->rank > 0) {
-      gint rank = factory->rank;
-      CLASS (xmlregistry)->save_func (xmlregistry, "<rank>%d</rank>\n", rank);
-    }	
-    
     templates = factory->padtemplates;
 
     while (templates) {
@@ -1620,7 +1613,7 @@ gst_xml_registry_rebuild (GstRegistry *registry)
   while (walk) {
     gchar *path = (gchar *) walk->data;
     
-    GST_INFO (GST_CAT_PLUGIN_LOADING, 
+    GST_CAT_INFO (GST_CAT_PLUGIN_LOADING, 
 	      "Rebuilding registry %p in directory %s...", registry, path);
 
     plugins = g_list_concat (plugins, 
@@ -1661,7 +1654,7 @@ gst_xml_registry_rebuild (GstRegistry *registry)
       g_warning ("Bizarre behavior: plugin %s actually loaded", 
 	         ((GstPlugin *) walk->data)->filename);
     } else {
-      GST_INFO (GST_CAT_PLUGIN_LOADING, "Plugin %s failed to load: %s", 
+      GST_CAT_INFO (GST_CAT_PLUGIN_LOADING, "Plugin %s failed to load: %s", 
                 ((GstPlugin *) walk->data)->filename, error->message);
       g_print ("Plugin %s failed to load\n",
                 ((GstPlugin *) walk->data)->filename);
