@@ -20,6 +20,10 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
+
 #include <gst/gst.h>
 
 #include <stdlib.h>
@@ -29,8 +33,7 @@
 #include <unistd.h>
 #include <string.h>
 #include <errno.h>
-
-#include "config.h"
+#include <locale.h>
 
 extern gboolean _gst_registry_auto_load;
 static gint num_features = 0;
@@ -46,11 +49,34 @@ plugin_added_func (GstRegistry *registry, GstPlugin *plugin, gpointer user_data)
   num_plugins++;
 }
 
+static void
+spawn_all_in_dir(const char *dirname)
+{
+  char *argv[2] = { NULL, NULL };
+  GDir *dir;
+  const char *file;
+
+  /* g_print("spawning all in %s\n", dirname); */
+
+  dir = g_dir_open (dirname, 0, NULL);
+  if (dir == NULL) return;
+
+  while( (file = g_dir_read_name (dir)) ){
+    argv[0] = g_build_filename(dirname, file);
+    g_print("running %s\n",argv[0]);
+    g_spawn_sync (NULL, argv, NULL, G_SPAWN_FILE_AND_ARGV_ZERO, NULL, NULL, NULL, NULL, NULL, NULL);
+    g_free(argv[0]);
+  }
+  g_dir_close(dir);
+}
+
 int main (int argc,char *argv[])
 {
   GList *registries;
   GList *path_spill = NULL; /* used for path spill from failing registries */
 
+  setlocale(LC_ALL, "");
+
     /* Init gst */
   _gst_registry_auto_load = FALSE;
   gst_init (&argc, &argv);
@@ -60,6 +86,10 @@ int main (int argc,char *argv[])
 
   while (registries) {
     GstRegistry *registry = GST_REGISTRY (registries->data);
+    GList *dir_list;
+    GList *iter;
+    char *dir;
+
     if (path_spill)
     {
       GList *iter;
@@ -81,7 +111,10 @@ int main (int argc,char *argv[])
 		      G_CALLBACK (plugin_added_func), NULL);
 
     if (registry->flags & GST_REGISTRY_WRITABLE) {
-      g_print ("rebuilding %s\n", registry->name);
+      char *location;
+      g_object_get (registry, "location", &location, NULL);
+      g_print ("rebuilding %s (%s)\n", registry->name, location);
+      g_free (location);
       gst_registry_rebuild (registry);
       gst_registry_save (registry);
     }
@@ -96,6 +129,15 @@ int main (int argc,char *argv[])
 	g_assert (path_spill != NULL);
       }
     }
+
+    dir_list = gst_registry_get_path_list(registry);
+    for(iter = dir_list; iter; iter = iter->next) {
+      dir = g_build_filename((const char *)iter->data, "register-scripts");
+      spawn_all_in_dir(dir);
+      g_free(dir);
+    }
+    g_list_free(dir_list);
+
     registries = g_list_next (registries);
   }
 
